from typing import List

from tecton._internals import metadata_service
from tecton._internals.sdk_decorators import sdk_public_method
from tecton_proto.metadataservice.metadata_service__client_pb2 import ListWorkspacesRequest


@sdk_public_method
def list_workspaces() -> List[str]:
    """
    Returns a list of the names of all registered Workspaces.

    :return: A list of strings.
    """
    request = ListWorkspacesRequest()
    response = metadata_service.instance().ListWorkspaces(request)
    return sorted([workspace.name for workspace in response.workspaces])



from tecton_proto.data import state_update__client_pb2 as _state_update__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DATA_SOURCE: TectonObjectType
DESCRIPTOR: _descriptor.FileDescriptor
ENTITY: TectonObjectType
FEATURE_SERVICE: TectonObjectType
FEATURE_VIEW: TectonObjectType
RESOURCE_PROVIDER: TectonObjectType
SERVER_GROUP: TectonObjectType
TECTON_OBJECT_TYPE_UNKNOWN: TectonObjectType
TRANSFORMATION: TectonObjectType

class TectonObjectDiff(_message.Message):
    __slots__ = ["object_metadata", "transition_side_effects", "transition_type", "warnings"]
    OBJECT_METADATA_FIELD_NUMBER: _ClassVar[int]
    TRANSITION_SIDE_EFFECTS_FIELD_NUMBER: _ClassVar[int]
    TRANSITION_TYPE_FIELD_NUMBER: _ClassVar[int]
    WARNINGS_FIELD_NUMBER: _ClassVar[int]
    object_metadata: TectonObjectMetadata
    transition_side_effects: _state_update__client_pb2.FcoTransitionSideEffects
    transition_type: _state_update__client_pb2.FcoTransitionType
    warnings: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, transition_type: _Optional[_Union[_state_update__client_pb2.FcoTransitionType, str]] = ..., transition_side_effects: _Optional[_Union[_state_update__client_pb2.FcoTransitionSideEffects, _Mapping]] = ..., object_metadata: _Optional[_Union[TectonObjectMetadata, _Mapping]] = ..., warnings: _Optional[_Iterable[str]] = ...) -> None: ...

class TectonObjectMetadata(_message.Message):
    __slots__ = ["description", "name", "object_type", "owner", "tags"]
    class TagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    OBJECT_TYPE_FIELD_NUMBER: _ClassVar[int]
    OWNER_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    description: str
    name: str
    object_type: TectonObjectType
    owner: str
    tags: _containers.ScalarMap[str, str]
    def __init__(self, name: _Optional[str] = ..., object_type: _Optional[_Union[TectonObjectType, str]] = ..., owner: _Optional[str] = ..., description: _Optional[str] = ..., tags: _Optional[_Mapping[str, str]] = ...) -> None: ...

class TectonRepoDiffSummary(_message.Message):
    __slots__ = ["object_diffs", "plan_id"]
    OBJECT_DIFFS_FIELD_NUMBER: _ClassVar[int]
    PLAN_ID_FIELD_NUMBER: _ClassVar[int]
    object_diffs: _containers.RepeatedCompositeFieldContainer[TectonObjectDiff]
    plan_id: str
    def __init__(self, object_diffs: _Optional[_Iterable[_Union[TectonObjectDiff, _Mapping]]] = ..., plan_id: _Optional[str] = ...) -> None: ...

class TectonObjectType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor
RESOURCE_TYPE_ACCESS: ResourceType
RESOURCE_TYPE_CLIENT_SECRET: ResourceType
RESOURCE_TYPE_CONFIG: ResourceType
RESOURCE_TYPE_DATA: ResourceType
RESOURCE_TYPE_DATASET: ResourceType
RESOURCE_TYPE_DEPLOYMENT_SETTINGS: ResourceType
RESOURCE_TYPE_FEATURES: ResourceType
RESOURCE_TYPE_MATERIALIZATION_STATE: ResourceType
RESOURCE_TYPE_MEMBERS: ResourceType
RESOURCE_TYPE_METRIC: ResourceType
RESOURCE_TYPE_ORGANIZATION: ResourceType
RESOURCE_TYPE_PLATFORM: ResourceType
RESOURCE_TYPE_PRINCIPAL_GROUP: ResourceType
RESOURCE_TYPE_PRINCIPAL_GROUPS_MEMBERSHIPS: ResourceType
RESOURCE_TYPE_SECRET: ResourceType
RESOURCE_TYPE_SECRET_SCOPE: ResourceType
RESOURCE_TYPE_SERVICE_ACCOUNT: ResourceType
RESOURCE_TYPE_UNSPECIFIED: ResourceType
RESOURCE_TYPE_USER: ResourceType
RESOURCE_TYPE_WORKSPACE: ResourceType

class ResourceType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import struct_pb2 as _struct_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from tecton_proto.args import basic_info__client_pb2 as _basic_info__client_pb2
from tecton_proto.args import data_source__client_pb2 as _data_source__client_pb2
from tecton_proto.args import diff_options__client_pb2 as _diff_options__client_pb2
from tecton_proto.args import pipeline__client_pb2 as _pipeline__client_pb2
from tecton_proto.args import server_group__client_pb2 as _server_group__client_pb2
from tecton_proto.common import analytics_options__client_pb2 as _analytics_options__client_pb2
from tecton_proto.common import calculation_node__client_pb2 as _calculation_node__client_pb2
from tecton_proto.common import compute_mode__client_pb2 as _compute_mode__client_pb2
from tecton_proto.common import data_source_type__client_pb2 as _data_source_type__client_pb2
from tecton_proto.common import data_type__client_pb2 as _data_type__client_pb2
from tecton_proto.common import framework_version__client_pb2 as _framework_version__client_pb2
from tecton_proto.common import id__client_pb2 as _id__client_pb2
from tecton_proto.common import python_version__client_pb2 as _python_version__client_pb2
from tecton_proto.common import schema__client_pb2 as _schema__client_pb2
from tecton_proto.common import secret__client_pb2 as _secret__client_pb2
from tecton_proto.common import spark_schema__client_pb2 as _spark_schema__client_pb2
from tecton_proto.common import time_window__client_pb2 as _time_window__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

AGGREGATION_MODE_LATEST_EVENT_TIME: AggregationLeadingEdge
AGGREGATION_MODE_UNSPECIFIED: AggregationLeadingEdge
AGGREGATION_MODE_WALL_CLOCK_TIME: AggregationLeadingEdge
BACKFILL_CONFIG_MODE_MULTIPLE_BATCH_SCHEDULE_INTERVALS_PER_JOB: BackfillConfigMode
BACKFILL_CONFIG_MODE_SINGLE_BATCH_SCHEDULE_INTERVAL_PER_JOB: BackfillConfigMode
BACKFILL_CONFIG_MODE_UNSPECIFIED: BackfillConfigMode
BATCH_TRIGGER_TYPE_MANUAL: BatchTriggerType
BATCH_TRIGGER_TYPE_NO_BATCH_MATERIALIZATION: BatchTriggerType
BATCH_TRIGGER_TYPE_SCHEDULED: BatchTriggerType
BATCH_TRIGGER_TYPE_UNSPECIFIED: BatchTriggerType
DESCRIPTOR: _descriptor.FileDescriptor
FEATURE_STORE_FORMAT_VERSION_DEFAULT: FeatureStoreFormatVersion
FEATURE_STORE_FORMAT_VERSION_MAX: FeatureStoreFormatVersion
FEATURE_STORE_FORMAT_VERSION_ONLINE_STORE_TTL_DELETION_ENABLED: FeatureStoreFormatVersion
FEATURE_STORE_FORMAT_VERSION_TIME_NANOSECONDS: FeatureStoreFormatVersion
FEATURE_STORE_FORMAT_VERSION_TTL_FIELD: FeatureStoreFormatVersion
FEATURE_VIEW_TYPE_FEATURE_TABLE: FeatureViewType
FEATURE_VIEW_TYPE_FWV5_FEATURE_VIEW: FeatureViewType
FEATURE_VIEW_TYPE_PROMPT: FeatureViewType
FEATURE_VIEW_TYPE_REALTIME: FeatureViewType
FEATURE_VIEW_TYPE_UNSPECIFIED: FeatureViewType
STREAM_PROCESSING_MODE_CONTINUOUS: StreamProcessingMode
STREAM_PROCESSING_MODE_TIME_INTERVAL: StreamProcessingMode
STREAM_PROCESSING_MODE_UNSPECIFIED: StreamProcessingMode

class Attribute(_message.Message):
    __slots__ = ["column_dtype", "description", "name", "tags"]
    class TagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    COLUMN_DTYPE_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    column_dtype: _data_type__client_pb2.DataType
    description: str
    name: str
    tags: _containers.ScalarMap[str, str]
    def __init__(self, name: _Optional[str] = ..., column_dtype: _Optional[_Union[_data_type__client_pb2.DataType, _Mapping]] = ..., description: _Optional[str] = ..., tags: _Optional[_Mapping[str, str]] = ...) -> None: ...

class BackfillConfig(_message.Message):
    __slots__ = ["mode"]
    MODE_FIELD_NUMBER: _ClassVar[int]
    mode: BackfillConfigMode
    def __init__(self, mode: _Optional[_Union[BackfillConfigMode, str]] = ...) -> None: ...

class BigtableOnlineStore(_message.Message):
    __slots__ = ["enabled", "instance_id", "project_id"]
    ENABLED_FIELD_NUMBER: _ClassVar[int]
    INSTANCE_ID_FIELD_NUMBER: _ClassVar[int]
    PROJECT_ID_FIELD_NUMBER: _ClassVar[int]
    enabled: bool
    instance_id: str
    project_id: str
    def __init__(self, enabled: bool = ..., project_id: _Optional[str] = ..., instance_id: _Optional[str] = ...) -> None: ...

class CacheConfig(_message.Message):
    __slots__ = ["max_age_seconds"]
    MAX_AGE_SECONDS_FIELD_NUMBER: _ClassVar[int]
    max_age_seconds: int
    def __init__(self, max_age_seconds: _Optional[int] = ...) -> None: ...

class Calculation(_message.Message):
    __slots__ = ["abstract_syntax_tree_root", "column_dtype", "description", "expr", "name", "tags"]
    class TagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    ABSTRACT_SYNTAX_TREE_ROOT_FIELD_NUMBER: _ClassVar[int]
    COLUMN_DTYPE_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    EXPR_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    abstract_syntax_tree_root: _calculation_node__client_pb2.AbstractSyntaxTreeNode
    column_dtype: _data_type__client_pb2.DataType
    description: str
    expr: str
    name: str
    tags: _containers.ScalarMap[str, str]
    def __init__(self, name: _Optional[str] = ..., expr: _Optional[str] = ..., column_dtype: _Optional[_Union[_data_type__client_pb2.DataType, _Mapping]] = ..., description: _Optional[str] = ..., tags: _Optional[_Mapping[str, str]] = ..., abstract_syntax_tree_root: _Optional[_Union[_calculation_node__client_pb2.AbstractSyntaxTreeNode, _Mapping]] = ...) -> None: ...

class ClusterConfig(_message.Message):
    __slots__ = ["existing_cluster", "implicit_config", "json_databricks", "json_dataproc", "json_emr", "new_databricks", "new_emr", "rift"]
    EXISTING_CLUSTER_FIELD_NUMBER: _ClassVar[int]
    IMPLICIT_CONFIG_FIELD_NUMBER: _ClassVar[int]
    JSON_DATABRICKS_FIELD_NUMBER: _ClassVar[int]
    JSON_DATAPROC_FIELD_NUMBER: _ClassVar[int]
    JSON_EMR_FIELD_NUMBER: _ClassVar[int]
    NEW_DATABRICKS_FIELD_NUMBER: _ClassVar[int]
    NEW_EMR_FIELD_NUMBER: _ClassVar[int]
    RIFT_FIELD_NUMBER: _ClassVar[int]
    existing_cluster: ExistingClusterConfig
    implicit_config: DefaultClusterConfig
    json_databricks: JsonClusterConfig
    json_dataproc: JsonClusterConfig
    json_emr: JsonClusterConfig
    new_databricks: NewClusterConfig
    new_emr: NewClusterConfig
    rift: RiftClusterConfig
    def __init__(self, existing_cluster: _Optional[_Union[ExistingClusterConfig, _Mapping]] = ..., new_databricks: _Optional[_Union[NewClusterConfig, _Mapping]] = ..., new_emr: _Optional[_Union[NewClusterConfig, _Mapping]] = ..., implicit_config: _Optional[_Union[DefaultClusterConfig, _Mapping]] = ..., json_databricks: _Optional[_Union[JsonClusterConfig, _Mapping]] = ..., json_emr: _Optional[_Union[JsonClusterConfig, _Mapping]] = ..., json_dataproc: _Optional[_Union[JsonClusterConfig, _Mapping]] = ..., rift: _Optional[_Union[RiftClusterConfig, _Mapping]] = ...) -> None: ...

class DataQualityConfig(_message.Message):
    __slots__ = ["data_quality_enabled", "skip_default_expectations"]
    DATA_QUALITY_ENABLED_FIELD_NUMBER: _ClassVar[int]
    SKIP_DEFAULT_EXPECTATIONS_FIELD_NUMBER: _ClassVar[int]
    data_quality_enabled: bool
    skip_default_expectations: bool
    def __init__(self, data_quality_enabled: bool = ..., skip_default_expectations: bool = ...) -> None: ...

class DefaultClusterConfig(_message.Message):
    __slots__ = ["databricks_spark_version", "emr_spark_version", "tecton_compute_instance_type"]
    DATABRICKS_SPARK_VERSION_FIELD_NUMBER: _ClassVar[int]
    EMR_SPARK_VERSION_FIELD_NUMBER: _ClassVar[int]
    TECTON_COMPUTE_INSTANCE_TYPE_FIELD_NUMBER: _ClassVar[int]
    databricks_spark_version: str
    emr_spark_version: str
    tecton_compute_instance_type: str
    def __init__(self, databricks_spark_version: _Optional[str] = ..., emr_spark_version: _Optional[str] = ..., tecton_compute_instance_type: _Optional[str] = ...) -> None: ...

class DeltaConfig(_message.Message):
    __slots__ = ["time_partition_size"]
    TIME_PARTITION_SIZE_FIELD_NUMBER: _ClassVar[int]
    time_partition_size: _duration_pb2.Duration
    def __init__(self, time_partition_size: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ...) -> None: ...

class DynamoDbOnlineStore(_message.Message):
    __slots__ = ["enabled", "replica_regions"]
    ENABLED_FIELD_NUMBER: _ClassVar[int]
    REPLICA_REGIONS_FIELD_NUMBER: _ClassVar[int]
    enabled: bool
    replica_regions: NullableStringList
    def __init__(self, enabled: bool = ..., replica_regions: _Optional[_Union[NullableStringList, _Mapping]] = ...) -> None: ...

class Embedding(_message.Message):
    __slots__ = ["column", "column_dtype", "description", "model", "name", "tags"]
    class TagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    COLUMN_DTYPE_FIELD_NUMBER: _ClassVar[int]
    COLUMN_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    MODEL_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    column: str
    column_dtype: _data_type__client_pb2.DataType
    description: str
    model: str
    name: str
    tags: _containers.ScalarMap[str, str]
    def __init__(self, name: _Optional[str] = ..., column: _Optional[str] = ..., column_dtype: _Optional[_Union[_data_type__client_pb2.DataType, _Mapping]] = ..., model: _Optional[str] = ..., description: _Optional[str] = ..., tags: _Optional[_Mapping[str, str]] = ...) -> None: ...

class EntityKeyOverride(_message.Message):
    __slots__ = ["entity_id", "join_keys"]
    ENTITY_ID_FIELD_NUMBER: _ClassVar[int]
    JOIN_KEYS_FIELD_NUMBER: _ClassVar[int]
    entity_id: _id__client_pb2.Id
    join_keys: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, entity_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., join_keys: _Optional[_Iterable[str]] = ...) -> None: ...

class ExistingClusterConfig(_message.Message):
    __slots__ = ["existing_cluster_id"]
    EXISTING_CLUSTER_ID_FIELD_NUMBER: _ClassVar[int]
    existing_cluster_id: str
    def __init__(self, existing_cluster_id: _Optional[str] = ...) -> None: ...

class FeatureAggregation(_message.Message):
    __slots__ = ["batch_sawtooth_tile_size", "column", "column_dtype", "description", "function", "function_params", "lifetime_window", "name", "tags", "time_window", "time_window_legacy", "time_window_series"]
    class FunctionParamsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: ParamValue
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[ParamValue, _Mapping]] = ...) -> None: ...
    class TagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    BATCH_SAWTOOTH_TILE_SIZE_FIELD_NUMBER: _ClassVar[int]
    COLUMN_DTYPE_FIELD_NUMBER: _ClassVar[int]
    COLUMN_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    FUNCTION_FIELD_NUMBER: _ClassVar[int]
    FUNCTION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    LIFETIME_WINDOW_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    TIME_WINDOW_FIELD_NUMBER: _ClassVar[int]
    TIME_WINDOW_LEGACY_FIELD_NUMBER: _ClassVar[int]
    TIME_WINDOW_SERIES_FIELD_NUMBER: _ClassVar[int]
    batch_sawtooth_tile_size: _duration_pb2.Duration
    column: str
    column_dtype: _data_type__client_pb2.DataType
    description: str
    function: str
    function_params: _containers.MessageMap[str, ParamValue]
    lifetime_window: _time_window__client_pb2.LifetimeWindow
    name: str
    tags: _containers.ScalarMap[str, str]
    time_window: TimeWindow
    time_window_legacy: _duration_pb2.Duration
    time_window_series: TimeWindowSeries
    def __init__(self, column: _Optional[str] = ..., function: _Optional[str] = ..., function_params: _Optional[_Mapping[str, ParamValue]] = ..., time_window_legacy: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., name: _Optional[str] = ..., time_window: _Optional[_Union[TimeWindow, _Mapping]] = ..., lifetime_window: _Optional[_Union[_time_window__client_pb2.LifetimeWindow, _Mapping]] = ..., time_window_series: _Optional[_Union[TimeWindowSeries, _Mapping]] = ..., column_dtype: _Optional[_Union[_data_type__client_pb2.DataType, _Mapping]] = ..., batch_sawtooth_tile_size: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., description: _Optional[str] = ..., tags: _Optional[_Mapping[str, str]] = ...) -> None: ...

class FeatureTableArgs(_message.Message):
    __slots__ = ["attributes", "batch_compute", "monitoring", "offline_store", "offline_store_legacy", "online_store", "schema", "serving_ttl", "tecton_materialization_runtime", "timestamp_field"]
    ATTRIBUTES_FIELD_NUMBER: _ClassVar[int]
    BATCH_COMPUTE_FIELD_NUMBER: _ClassVar[int]
    MONITORING_FIELD_NUMBER: _ClassVar[int]
    OFFLINE_STORE_FIELD_NUMBER: _ClassVar[int]
    OFFLINE_STORE_LEGACY_FIELD_NUMBER: _ClassVar[int]
    ONLINE_STORE_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_FIELD_NUMBER: _ClassVar[int]
    SERVING_TTL_FIELD_NUMBER: _ClassVar[int]
    TECTON_MATERIALIZATION_RUNTIME_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_FIELD_FIELD_NUMBER: _ClassVar[int]
    attributes: _containers.RepeatedCompositeFieldContainer[Attribute]
    batch_compute: ClusterConfig
    monitoring: MonitoringConfig
    offline_store: OfflineStoreConfig
    offline_store_legacy: OfflineFeatureStoreConfig
    online_store: OnlineStoreConfig
    schema: _spark_schema__client_pb2.SparkSchema
    serving_ttl: _duration_pb2.Duration
    tecton_materialization_runtime: str
    timestamp_field: str
    def __init__(self, schema: _Optional[_Union[_spark_schema__client_pb2.SparkSchema, _Mapping]] = ..., serving_ttl: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., offline_store_legacy: _Optional[_Union[OfflineFeatureStoreConfig, _Mapping]] = ..., offline_store: _Optional[_Union[OfflineStoreConfig, _Mapping]] = ..., online_store: _Optional[_Union[OnlineStoreConfig, _Mapping]] = ..., batch_compute: _Optional[_Union[ClusterConfig, _Mapping]] = ..., monitoring: _Optional[_Union[MonitoringConfig, _Mapping]] = ..., tecton_materialization_runtime: _Optional[str] = ..., attributes: _Optional[_Iterable[_Union[Attribute, _Mapping]]] = ..., timestamp_field: _Optional[str] = ...) -> None: ...

class FeatureViewArgs(_message.Message):
    __slots__ = ["batch_compute_mode", "cache_config", "context_parameter_name", "data_quality_config", "entities", "feature_table_args", "feature_view_id", "feature_view_type", "forced_materialized_schema", "forced_view_schema", "info", "materialized_feature_view_args", "offline_enabled", "online_enabled", "online_serving_index", "options", "pipeline", "prevent_destroy", "prompt_args", "realtime_args", "resource_providers", "secrets", "version"]
    class OptionsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    class ResourceProvidersEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: _id__client_pb2.Id
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ...) -> None: ...
    class SecretsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: _secret__client_pb2.SecretReference
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[_secret__client_pb2.SecretReference, _Mapping]] = ...) -> None: ...
    BATCH_COMPUTE_MODE_FIELD_NUMBER: _ClassVar[int]
    CACHE_CONFIG_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_PARAMETER_NAME_FIELD_NUMBER: _ClassVar[int]
    DATA_QUALITY_CONFIG_FIELD_NUMBER: _ClassVar[int]
    ENTITIES_FIELD_NUMBER: _ClassVar[int]
    FEATURE_TABLE_ARGS_FIELD_NUMBER: _ClassVar[int]
    FEATURE_VIEW_ID_FIELD_NUMBER: _ClassVar[int]
    FEATURE_VIEW_TYPE_FIELD_NUMBER: _ClassVar[int]
    FORCED_MATERIALIZED_SCHEMA_FIELD_NUMBER: _ClassVar[int]
    FORCED_VIEW_SCHEMA_FIELD_NUMBER: _ClassVar[int]
    INFO_FIELD_NUMBER: _ClassVar[int]
    MATERIALIZED_FEATURE_VIEW_ARGS_FIELD_NUMBER: _ClassVar[int]
    OFFLINE_ENABLED_FIELD_NUMBER: _ClassVar[int]
    ONLINE_ENABLED_FIELD_NUMBER: _ClassVar[int]
    ONLINE_SERVING_INDEX_FIELD_NUMBER: _ClassVar[int]
    OPTIONS_FIELD_NUMBER: _ClassVar[int]
    PIPELINE_FIELD_NUMBER: _ClassVar[int]
    PREVENT_DESTROY_FIELD_NUMBER: _ClassVar[int]
    PROMPT_ARGS_FIELD_NUMBER: _ClassVar[int]
    REALTIME_ARGS_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_PROVIDERS_FIELD_NUMBER: _ClassVar[int]
    SECRETS_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    batch_compute_mode: _compute_mode__client_pb2.BatchComputeMode
    cache_config: CacheConfig
    context_parameter_name: str
    data_quality_config: DataQualityConfig
    entities: _containers.RepeatedCompositeFieldContainer[EntityKeyOverride]
    feature_table_args: FeatureTableArgs
    feature_view_id: _id__client_pb2.Id
    feature_view_type: FeatureViewType
    forced_materialized_schema: _spark_schema__client_pb2.SparkSchema
    forced_view_schema: _spark_schema__client_pb2.SparkSchema
    info: _basic_info__client_pb2.BasicInfo
    materialized_feature_view_args: MaterializedFeatureViewArgs
    offline_enabled: bool
    online_enabled: bool
    online_serving_index: _containers.RepeatedScalarFieldContainer[str]
    options: _containers.ScalarMap[str, str]
    pipeline: _pipeline__client_pb2.Pipeline
    prevent_destroy: bool
    prompt_args: PromptArgs
    realtime_args: RealtimeArgs
    resource_providers: _containers.MessageMap[str, _id__client_pb2.Id]
    secrets: _containers.MessageMap[str, _secret__client_pb2.SecretReference]
    version: _framework_version__client_pb2.FrameworkVersion
    def __init__(self, feature_view_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., feature_view_type: _Optional[_Union[FeatureViewType, str]] = ..., info: _Optional[_Union[_basic_info__client_pb2.BasicInfo, _Mapping]] = ..., version: _Optional[_Union[_framework_version__client_pb2.FrameworkVersion, str]] = ..., prevent_destroy: bool = ..., options: _Optional[_Mapping[str, str]] = ..., cache_config: _Optional[_Union[CacheConfig, _Mapping]] = ..., entities: _Optional[_Iterable[_Union[EntityKeyOverride, _Mapping]]] = ..., resource_providers: _Optional[_Mapping[str, _id__client_pb2.Id]] = ..., materialized_feature_view_args: _Optional[_Union[MaterializedFeatureViewArgs, _Mapping]] = ..., realtime_args: _Optional[_Union[RealtimeArgs, _Mapping]] = ..., feature_table_args: _Optional[_Union[FeatureTableArgs, _Mapping]] = ..., prompt_args: _Optional[_Union[PromptArgs, _Mapping]] = ..., context_parameter_name: _Optional[str] = ..., secrets: _Optional[_Mapping[str, _secret__client_pb2.SecretReference]] = ..., online_serving_index: _Optional[_Iterable[str]] = ..., online_enabled: bool = ..., offline_enabled: bool = ..., batch_compute_mode: _Optional[_Union[_compute_mode__client_pb2.BatchComputeMode, str]] = ..., pipeline: _Optional[_Union[_pipeline__client_pb2.Pipeline, _Mapping]] = ..., data_quality_config: _Optional[_Union[DataQualityConfig, _Mapping]] = ..., forced_view_schema: _Optional[_Union[_spark_schema__client_pb2.SparkSchema, _Mapping]] = ..., forced_materialized_schema: _Optional[_Union[_spark_schema__client_pb2.SparkSchema, _Mapping]] = ...) -> None: ...

class Inference(_message.Message):
    __slots__ = ["description", "input_columns", "model", "name", "tags"]
    class TagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    INPUT_COLUMNS_FIELD_NUMBER: _ClassVar[int]
    MODEL_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    description: str
    input_columns: _containers.RepeatedCompositeFieldContainer[_schema__client_pb2.Field]
    model: str
    name: str
    tags: _containers.ScalarMap[str, str]
    def __init__(self, input_columns: _Optional[_Iterable[_Union[_schema__client_pb2.Field, _Mapping]]] = ..., name: _Optional[str] = ..., model: _Optional[str] = ..., description: _Optional[str] = ..., tags: _Optional[_Mapping[str, str]] = ...) -> None: ...

class JsonClusterConfig(_message.Message):
    __slots__ = ["json"]
    JSON_FIELD_NUMBER: _ClassVar[int]
    json: _struct_pb2.Struct
    def __init__(self, json: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ...) -> None: ...

class MaterializedFeatureViewArgs(_message.Message):
    __slots__ = ["aggregation_interval", "aggregation_leading_edge", "aggregation_secondary_key", "aggregations", "attributes", "batch_compute", "batch_schedule", "batch_trigger", "compaction_enabled", "data_source_type", "embeddings", "environment", "feature_start_time", "feature_store_format_version", "incremental_backfills", "inferences", "lifetime_start_time", "manual_trigger_backfill_end_time", "max_backfill_interval", "monitoring", "offline_store", "offline_store_legacy", "online_store", "output_stream", "run_transformation_validation", "schema", "secondary_key_output_columns", "secrets", "serving_ttl", "stream_compute", "stream_processing_mode", "stream_tile_size", "stream_tiling_enabled", "tecton_materialization_runtime", "timestamp_field", "transform_server_group"]
    class SecretsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: _secret__client_pb2.SecretReference
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[_secret__client_pb2.SecretReference, _Mapping]] = ...) -> None: ...
    AGGREGATIONS_FIELD_NUMBER: _ClassVar[int]
    AGGREGATION_INTERVAL_FIELD_NUMBER: _ClassVar[int]
    AGGREGATION_LEADING_EDGE_FIELD_NUMBER: _ClassVar[int]
    AGGREGATION_SECONDARY_KEY_FIELD_NUMBER: _ClassVar[int]
    ATTRIBUTES_FIELD_NUMBER: _ClassVar[int]
    BATCH_COMPUTE_FIELD_NUMBER: _ClassVar[int]
    BATCH_SCHEDULE_FIELD_NUMBER: _ClassVar[int]
    BATCH_TRIGGER_FIELD_NUMBER: _ClassVar[int]
    COMPACTION_ENABLED_FIELD_NUMBER: _ClassVar[int]
    DATA_SOURCE_TYPE_FIELD_NUMBER: _ClassVar[int]
    EMBEDDINGS_FIELD_NUMBER: _ClassVar[int]
    ENVIRONMENT_FIELD_NUMBER: _ClassVar[int]
    FEATURE_START_TIME_FIELD_NUMBER: _ClassVar[int]
    FEATURE_STORE_FORMAT_VERSION_FIELD_NUMBER: _ClassVar[int]
    INCREMENTAL_BACKFILLS_FIELD_NUMBER: _ClassVar[int]
    INFERENCES_FIELD_NUMBER: _ClassVar[int]
    LIFETIME_START_TIME_FIELD_NUMBER: _ClassVar[int]
    MANUAL_TRIGGER_BACKFILL_END_TIME_FIELD_NUMBER: _ClassVar[int]
    MAX_BACKFILL_INTERVAL_FIELD_NUMBER: _ClassVar[int]
    MONITORING_FIELD_NUMBER: _ClassVar[int]
    OFFLINE_STORE_FIELD_NUMBER: _ClassVar[int]
    OFFLINE_STORE_LEGACY_FIELD_NUMBER: _ClassVar[int]
    ONLINE_STORE_FIELD_NUMBER: _ClassVar[int]
    OUTPUT_STREAM_FIELD_NUMBER: _ClassVar[int]
    RUN_TRANSFORMATION_VALIDATION_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_FIELD_NUMBER: _ClassVar[int]
    SECONDARY_KEY_OUTPUT_COLUMNS_FIELD_NUMBER: _ClassVar[int]
    SECRETS_FIELD_NUMBER: _ClassVar[int]
    SERVING_TTL_FIELD_NUMBER: _ClassVar[int]
    STREAM_COMPUTE_FIELD_NUMBER: _ClassVar[int]
    STREAM_PROCESSING_MODE_FIELD_NUMBER: _ClassVar[int]
    STREAM_TILE_SIZE_FIELD_NUMBER: _ClassVar[int]
    STREAM_TILING_ENABLED_FIELD_NUMBER: _ClassVar[int]
    TECTON_MATERIALIZATION_RUNTIME_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_FIELD_FIELD_NUMBER: _ClassVar[int]
    TRANSFORM_SERVER_GROUP_FIELD_NUMBER: _ClassVar[int]
    aggregation_interval: _duration_pb2.Duration
    aggregation_leading_edge: AggregationLeadingEdge
    aggregation_secondary_key: str
    aggregations: _containers.RepeatedCompositeFieldContainer[FeatureAggregation]
    attributes: _containers.RepeatedCompositeFieldContainer[Attribute]
    batch_compute: ClusterConfig
    batch_schedule: _duration_pb2.Duration
    batch_trigger: BatchTriggerType
    compaction_enabled: bool
    data_source_type: _data_source_type__client_pb2.DataSourceType
    embeddings: _containers.RepeatedCompositeFieldContainer[Embedding]
    environment: str
    feature_start_time: _timestamp_pb2.Timestamp
    feature_store_format_version: FeatureStoreFormatVersion
    incremental_backfills: bool
    inferences: _containers.RepeatedCompositeFieldContainer[Inference]
    lifetime_start_time: _timestamp_pb2.Timestamp
    manual_trigger_backfill_end_time: _timestamp_pb2.Timestamp
    max_backfill_interval: _duration_pb2.Duration
    monitoring: MonitoringConfig
    offline_store: OfflineStoreConfig
    offline_store_legacy: OfflineFeatureStoreConfig
    online_store: OnlineStoreConfig
    output_stream: OutputStream
    run_transformation_validation: bool
    schema: _schema__client_pb2.Schema
    secondary_key_output_columns: _containers.RepeatedCompositeFieldContainer[SecondaryKeyOutputColumn]
    secrets: _containers.MessageMap[str, _secret__client_pb2.SecretReference]
    serving_ttl: _duration_pb2.Duration
    stream_compute: ClusterConfig
    stream_processing_mode: StreamProcessingMode
    stream_tile_size: _duration_pb2.Duration
    stream_tiling_enabled: bool
    tecton_materialization_runtime: str
    timestamp_field: str
    transform_server_group: _server_group__client_pb2.ServerGroupReference
    def __init__(self, timestamp_field: _Optional[str] = ..., batch_schedule: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., feature_start_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., manual_trigger_backfill_end_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., max_backfill_interval: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., serving_ttl: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., offline_store_legacy: _Optional[_Union[OfflineFeatureStoreConfig, _Mapping]] = ..., offline_store: _Optional[_Union[OfflineStoreConfig, _Mapping]] = ..., batch_compute: _Optional[_Union[ClusterConfig, _Mapping]] = ..., stream_compute: _Optional[_Union[ClusterConfig, _Mapping]] = ..., monitoring: _Optional[_Union[MonitoringConfig, _Mapping]] = ..., data_source_type: _Optional[_Union[_data_source_type__client_pb2.DataSourceType, str]] = ..., online_store: _Optional[_Union[OnlineStoreConfig, _Mapping]] = ..., incremental_backfills: bool = ..., aggregation_interval: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., stream_processing_mode: _Optional[_Union[StreamProcessingMode, str]] = ..., aggregations: _Optional[_Iterable[_Union[FeatureAggregation, _Mapping]]] = ..., output_stream: _Optional[_Union[OutputStream, _Mapping]] = ..., batch_trigger: _Optional[_Union[BatchTriggerType, str]] = ..., schema: _Optional[_Union[_schema__client_pb2.Schema, _Mapping]] = ..., aggregation_secondary_key: _Optional[str] = ..., secondary_key_output_columns: _Optional[_Iterable[_Union[SecondaryKeyOutputColumn, _Mapping]]] = ..., run_transformation_validation: bool = ..., tecton_materialization_runtime: _Optional[str] = ..., lifetime_start_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., compaction_enabled: bool = ..., stream_tiling_enabled: bool = ..., stream_tile_size: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., environment: _Optional[str] = ..., transform_server_group: _Optional[_Union[_server_group__client_pb2.ServerGroupReference, _Mapping]] = ..., attributes: _Optional[_Iterable[_Union[Attribute, _Mapping]]] = ..., embeddings: _Optional[_Iterable[_Union[Embedding, _Mapping]]] = ..., inferences: _Optional[_Iterable[_Union[Inference, _Mapping]]] = ..., aggregation_leading_edge: _Optional[_Union[AggregationLeadingEdge, str]] = ..., feature_store_format_version: _Optional[_Union[FeatureStoreFormatVersion, str]] = ..., secrets: _Optional[_Mapping[str, _secret__client_pb2.SecretReference]] = ...) -> None: ...

class MonitoringConfig(_message.Message):
    __slots__ = ["alert_email", "expected_freshness", "monitor_freshness"]
    ALERT_EMAIL_FIELD_NUMBER: _ClassVar[int]
    EXPECTED_FRESHNESS_FIELD_NUMBER: _ClassVar[int]
    MONITOR_FRESHNESS_FIELD_NUMBER: _ClassVar[int]
    alert_email: str
    expected_freshness: _duration_pb2.Duration
    monitor_freshness: bool
    def __init__(self, monitor_freshness: bool = ..., expected_freshness: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., alert_email: _Optional[str] = ...) -> None: ...

class NewClusterConfig(_message.Message):
    __slots__ = ["extra_pip_dependencies", "first_on_demand", "instance_availability", "instance_type", "number_of_workers", "pinned_spark_version", "python_version", "root_volume_size_in_gb", "spark_config"]
    EXTRA_PIP_DEPENDENCIES_FIELD_NUMBER: _ClassVar[int]
    FIRST_ON_DEMAND_FIELD_NUMBER: _ClassVar[int]
    INSTANCE_AVAILABILITY_FIELD_NUMBER: _ClassVar[int]
    INSTANCE_TYPE_FIELD_NUMBER: _ClassVar[int]
    NUMBER_OF_WORKERS_FIELD_NUMBER: _ClassVar[int]
    PINNED_SPARK_VERSION_FIELD_NUMBER: _ClassVar[int]
    PYTHON_VERSION_FIELD_NUMBER: _ClassVar[int]
    ROOT_VOLUME_SIZE_IN_GB_FIELD_NUMBER: _ClassVar[int]
    SPARK_CONFIG_FIELD_NUMBER: _ClassVar[int]
    extra_pip_dependencies: _containers.RepeatedScalarFieldContainer[str]
    first_on_demand: int
    instance_availability: str
    instance_type: str
    number_of_workers: int
    pinned_spark_version: str
    python_version: _python_version__client_pb2.PythonVersion
    root_volume_size_in_gb: int
    spark_config: SparkConfig
    def __init__(self, instance_type: _Optional[str] = ..., instance_availability: _Optional[str] = ..., number_of_workers: _Optional[int] = ..., root_volume_size_in_gb: _Optional[int] = ..., extra_pip_dependencies: _Optional[_Iterable[str]] = ..., spark_config: _Optional[_Union[SparkConfig, _Mapping]] = ..., first_on_demand: _Optional[int] = ..., pinned_spark_version: _Optional[str] = ..., python_version: _Optional[_Union[_python_version__client_pb2.PythonVersion, str]] = ...) -> None: ...

class NullableStringList(_message.Message):
    __slots__ = ["values"]
    VALUES_FIELD_NUMBER: _ClassVar[int]
    values: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, values: _Optional[_Iterable[str]] = ...) -> None: ...

class OfflineFeatureStoreConfig(_message.Message):
    __slots__ = ["delta", "parquet", "subdirectory_override"]
    DELTA_FIELD_NUMBER: _ClassVar[int]
    PARQUET_FIELD_NUMBER: _ClassVar[int]
    SUBDIRECTORY_OVERRIDE_FIELD_NUMBER: _ClassVar[int]
    delta: DeltaConfig
    parquet: ParquetConfig
    subdirectory_override: str
    def __init__(self, parquet: _Optional[_Union[ParquetConfig, _Mapping]] = ..., delta: _Optional[_Union[DeltaConfig, _Mapping]] = ..., subdirectory_override: _Optional[str] = ...) -> None: ...

class OfflineStoreConfig(_message.Message):
    __slots__ = ["publish_full_features", "publish_start_time", "staging_table_format"]
    PUBLISH_FULL_FEATURES_FIELD_NUMBER: _ClassVar[int]
    PUBLISH_START_TIME_FIELD_NUMBER: _ClassVar[int]
    STAGING_TABLE_FORMAT_FIELD_NUMBER: _ClassVar[int]
    publish_full_features: bool
    publish_start_time: _timestamp_pb2.Timestamp
    staging_table_format: OfflineFeatureStoreConfig
    def __init__(self, staging_table_format: _Optional[_Union[OfflineFeatureStoreConfig, _Mapping]] = ..., publish_full_features: bool = ..., publish_start_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class OnlineStoreConfig(_message.Message):
    __slots__ = ["bigtable", "dynamo", "redis"]
    BIGTABLE_FIELD_NUMBER: _ClassVar[int]
    DYNAMO_FIELD_NUMBER: _ClassVar[int]
    REDIS_FIELD_NUMBER: _ClassVar[int]
    bigtable: BigtableOnlineStore
    dynamo: DynamoDbOnlineStore
    redis: RedisOnlineStore
    def __init__(self, dynamo: _Optional[_Union[DynamoDbOnlineStore, _Mapping]] = ..., redis: _Optional[_Union[RedisOnlineStore, _Mapping]] = ..., bigtable: _Optional[_Union[BigtableOnlineStore, _Mapping]] = ...) -> None: ...

class OutputStream(_message.Message):
    __slots__ = ["include_features", "kafka", "kinesis"]
    INCLUDE_FEATURES_FIELD_NUMBER: _ClassVar[int]
    KAFKA_FIELD_NUMBER: _ClassVar[int]
    KINESIS_FIELD_NUMBER: _ClassVar[int]
    include_features: bool
    kafka: _data_source__client_pb2.KafkaDataSourceArgs
    kinesis: _data_source__client_pb2.KinesisDataSourceArgs
    def __init__(self, include_features: bool = ..., kinesis: _Optional[_Union[_data_source__client_pb2.KinesisDataSourceArgs, _Mapping]] = ..., kafka: _Optional[_Union[_data_source__client_pb2.KafkaDataSourceArgs, _Mapping]] = ...) -> None: ...

class ParamValue(_message.Message):
    __slots__ = ["double_value", "int64_value"]
    DOUBLE_VALUE_FIELD_NUMBER: _ClassVar[int]
    INT64_VALUE_FIELD_NUMBER: _ClassVar[int]
    double_value: float
    int64_value: int
    def __init__(self, int64_value: _Optional[int] = ..., double_value: _Optional[float] = ...) -> None: ...

class ParquetConfig(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class PromptArgs(_message.Message):
    __slots__ = ["attributes", "environment"]
    ATTRIBUTES_FIELD_NUMBER: _ClassVar[int]
    ENVIRONMENT_FIELD_NUMBER: _ClassVar[int]
    attributes: _containers.RepeatedCompositeFieldContainer[Attribute]
    environment: str
    def __init__(self, environment: _Optional[str] = ..., attributes: _Optional[_Iterable[_Union[Attribute, _Mapping]]] = ...) -> None: ...

class RealtimeArgs(_message.Message):
    __slots__ = ["attributes", "calculations", "environments", "required_packages", "schema"]
    ATTRIBUTES_FIELD_NUMBER: _ClassVar[int]
    CALCULATIONS_FIELD_NUMBER: _ClassVar[int]
    ENVIRONMENTS_FIELD_NUMBER: _ClassVar[int]
    REQUIRED_PACKAGES_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_FIELD_NUMBER: _ClassVar[int]
    attributes: _containers.RepeatedCompositeFieldContainer[Attribute]
    calculations: _containers.RepeatedCompositeFieldContainer[Calculation]
    environments: _containers.RepeatedScalarFieldContainer[str]
    required_packages: _containers.RepeatedScalarFieldContainer[str]
    schema: _spark_schema__client_pb2.SparkSchema
    def __init__(self, schema: _Optional[_Union[_spark_schema__client_pb2.SparkSchema, _Mapping]] = ..., environments: _Optional[_Iterable[str]] = ..., required_packages: _Optional[_Iterable[str]] = ..., attributes: _Optional[_Iterable[_Union[Attribute, _Mapping]]] = ..., calculations: _Optional[_Iterable[_Union[Calculation, _Mapping]]] = ...) -> None: ...

class RedisOnlineStore(_message.Message):
    __slots__ = ["authentication_token", "enabled", "primary_endpoint"]
    AUTHENTICATION_TOKEN_FIELD_NUMBER: _ClassVar[int]
    ENABLED_FIELD_NUMBER: _ClassVar[int]
    PRIMARY_ENDPOINT_FIELD_NUMBER: _ClassVar[int]
    authentication_token: str
    enabled: bool
    primary_endpoint: str
    def __init__(self, primary_endpoint: _Optional[str] = ..., authentication_token: _Optional[str] = ..., enabled: bool = ...) -> None: ...

class RiftClusterConfig(_message.Message):
    __slots__ = ["instance_type"]
    INSTANCE_TYPE_FIELD_NUMBER: _ClassVar[int]
    instance_type: str
    def __init__(self, instance_type: _Optional[str] = ...) -> None: ...

class SecondaryKeyOutputColumn(_message.Message):
    __slots__ = ["lifetime_window", "name", "time_window", "time_window_series"]
    LIFETIME_WINDOW_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    TIME_WINDOW_FIELD_NUMBER: _ClassVar[int]
    TIME_WINDOW_SERIES_FIELD_NUMBER: _ClassVar[int]
    lifetime_window: _time_window__client_pb2.LifetimeWindow
    name: str
    time_window: TimeWindow
    time_window_series: TimeWindowSeries
    def __init__(self, time_window: _Optional[_Union[TimeWindow, _Mapping]] = ..., lifetime_window: _Optional[_Union[_time_window__client_pb2.LifetimeWindow, _Mapping]] = ..., time_window_series: _Optional[_Union[TimeWindowSeries, _Mapping]] = ..., name: _Optional[str] = ...) -> None: ...

class SparkConfig(_message.Message):
    __slots__ = ["spark_conf", "spark_driver_memory", "spark_driver_memory_overhead", "spark_executor_memory", "spark_executor_memory_overhead"]
    class SparkConfEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    SPARK_CONF_FIELD_NUMBER: _ClassVar[int]
    SPARK_DRIVER_MEMORY_FIELD_NUMBER: _ClassVar[int]
    SPARK_DRIVER_MEMORY_OVERHEAD_FIELD_NUMBER: _ClassVar[int]
    SPARK_EXECUTOR_MEMORY_FIELD_NUMBER: _ClassVar[int]
    SPARK_EXECUTOR_MEMORY_OVERHEAD_FIELD_NUMBER: _ClassVar[int]
    spark_conf: _containers.ScalarMap[str, str]
    spark_driver_memory: str
    spark_driver_memory_overhead: str
    spark_executor_memory: str
    spark_executor_memory_overhead: str
    def __init__(self, spark_driver_memory: _Optional[str] = ..., spark_executor_memory: _Optional[str] = ..., spark_driver_memory_overhead: _Optional[str] = ..., spark_executor_memory_overhead: _Optional[str] = ..., spark_conf: _Optional[_Mapping[str, str]] = ...) -> None: ...

class TimeWindow(_message.Message):
    __slots__ = ["offset", "window_duration"]
    OFFSET_FIELD_NUMBER: _ClassVar[int]
    WINDOW_DURATION_FIELD_NUMBER: _ClassVar[int]
    offset: _duration_pb2.Duration
    window_duration: _duration_pb2.Duration
    def __init__(self, window_duration: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., offset: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ...) -> None: ...

class TimeWindowSeries(_message.Message):
    __slots__ = ["series_end", "series_start", "step_size", "window_duration"]
    SERIES_END_FIELD_NUMBER: _ClassVar[int]
    SERIES_START_FIELD_NUMBER: _ClassVar[int]
    STEP_SIZE_FIELD_NUMBER: _ClassVar[int]
    WINDOW_DURATION_FIELD_NUMBER: _ClassVar[int]
    series_end: _duration_pb2.Duration
    series_start: _duration_pb2.Duration
    step_size: _duration_pb2.Duration
    window_duration: _duration_pb2.Duration
    def __init__(self, series_start: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., series_end: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., step_size: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., window_duration: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ...) -> None: ...

class FeatureViewType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class BackfillConfigMode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class AggregationLeadingEdge(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class StreamProcessingMode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class BatchTriggerType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class FeatureStoreFormatVersion(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



import logging

from tecton_core.compute_mode import ComputeMode
from tecton_core.query.retrieval_params import GetFeaturesForEventsParams
from tecton_core.query.retrieval_params import GetFeaturesInRangeParams
from tecton_core.query.rewrite import rewrite_tree_for_spine
from tecton_core.query_consts import valid_to
from tecton_materialization.common.dataset_generation import get_features_from_params
from tecton_materialization.common.task_params import get_features_params_from_task_params
from tecton_spark.offline_store import OfflineStoreWriterParams
from tecton_spark.offline_store import get_dataset_generation_writer
from tecton_spark.query import translate
from tecton_spark.query.translate import SparkDataFrame


logger = logging.getLogger(__name__)


def dataset_generation_from_params(spark, materialization_task_params):
    dataset_generation_params = materialization_task_params.dataset_generation_task_info.dataset_generation_parameters
    params = get_features_params_from_task_params(materialization_task_params, compute_mode=ComputeMode.SPARK)
    spark.conf.set(
        "spark.databricks.delta.commitInfo.userMetadata",
        f'{{"datasetPath":"{dataset_generation_params.result_path}"}}',
    )
    if isinstance(params, GetFeaturesForEventsParams):
        time_column = params.timestamp_key
        spine = SparkDataFrame(spark.read.parquet(params.events))
        qt = get_features_from_params(params, spine=spine)
    elif isinstance(params, GetFeaturesInRangeParams):
        time_column = valid_to()
        entities = SparkDataFrame(spark.read.parquet(params.entities)) if params.entities is not None else None
        qt = get_features_from_params(params, entities=entities)
    else:
        error = f"Unsupported params type: {type(params)}"
        raise ValueError(error)

    rewrite_tree_for_spine(qt)
    logger.info(f"Starting dataset generation job for '{params.fco.name}'")
    logger.info(f"QT: \n{qt.pretty_str(columns=True)}")

    spark_df = translate.spark_convert(qt, spark).to_dataframe(spark)
    writer_params = OfflineStoreWriterParams(
        s3_path=dataset_generation_params.result_path,
        always_store_anchor_column=False,
        time_column=time_column,
        join_key_columns=params.join_keys,
        is_continuous=False,
    )

    logger.info("Writing results to %s", dataset_generation_params.result_path)

    delta_writer = get_dataset_generation_writer(writer_params, spark)
    delta_writer.append_dataframe(spark_df)



import sys
import urllib
from typing import Any
from typing import Dict
from typing import Optional
from typing import Tuple
from typing import Union

import attrs
import requests

from tecton._internals import metadata_service
from tecton._internals.utils import cluster_url
from tecton.cli import printer
from tecton.cli import workspace
from tecton.cli.workspace_utils import switch_to_workspace
from tecton.identities import api_keys
from tecton.identities import okta
from tecton_core import conf
from tecton_core import http
from tecton_core.errors import TectonNotFoundError
from tecton_core.id_helper import IdHelper
from tecton_proto.auth.authorization_service__client_pb2 import GetIsAuthorizedRequest
from tecton_proto.auth.authorization_service__client_pb2 import Permission
from tecton_proto.auth.principal__client_pb2 import PrincipalType
from tecton_proto.auth.resource__client_pb2 import ResourceType


@attrs.frozen
class ServiceAccountProfile:
    id: str
    name: str
    description: str
    created_by: str
    is_active: bool
    obscured_key: str


@attrs.frozen
class LoginTempState:
    login_configs: Dict[str, Any]
    code_verifier: str
    redirect_uri: str
    tecton_url: str
    # between requesting the auth code and exchanging it to complete login, okta_auth_flow
    okta_auth_flow: okta.OktaAuthorizationFlow


notebook_login_temp_result = None


def _set_credentials_impl(tecton_api_service_url: Optional[str], tecton_api_key: Optional[str]) -> None:
    """
    Implementation for the deprecated set_credentials. Params must be Optional to still provide support in 1.0.

    :param tecton_api_service_url: Tecton API URL
    :param tecton_api_key: Tecton API Key
    """
    if tecton_api_key:
        conf.set("TECTON_API_KEY", tecton_api_key)
    if tecton_api_service_url:
        conf.validate_api_service_url(tecton_api_service_url)
        conf.set("API_SERVICE", tecton_api_service_url)


def _clear_credentials_impl() -> None:
    for key in (
        "TECTON_API_KEY",
        "API_SERVICE",
        "OAUTH_ACCESS_TOKEN",
        "OAUTH_REFRESH_TOKEN",
        "OAUTH_ACCESS_TOKEN_EXPIRATION",
    ):
        try:
            conf.unset(key)
        except KeyError:
            pass


def get_caller_identity() -> Union[ServiceAccountProfile, okta.UserProfile]:
    """
    Introspect the current User or API Key used to authenticate with Tecton.

    :return: The UserProfile or ServiceAccountProfile of the current User or API Key (respectively)
      if the introspection is successful.
    :raises SystemError: If the credentials are expired, invalid, or not configured.
    """
    print(f"Tecton Endpoint {cluster_url()}")

    user_profile = okta.get_user_profile()
    if user_profile:
        user_is_on_cluster = _check_user_can_read_org(user_profile.id)
        if user_is_on_cluster:
            return user_profile
        else:
            msg = (
                "Unable to authenticate using the configured user credentials. "
                "Please rerun `tecton login` with the CLI "
                "or `tecton.login(tecton_url=<tecton_url>)` in your notebook."
            )
            raise SystemError(msg)
    else:
        token = conf.get_or_none("TECTON_API_KEY")
        if token:
            token_invalid = False
            try:
                introspect_result = api_keys.introspect(token)
            except (PermissionError, TectonNotFoundError):
                token_invalid = True
            # raise this error outside of the except to avoid chaining errors, to avoid eyesore in notebook envs
            if token_invalid:
                msg = (
                    "Permissions error when introspecting the Tecton API key. "
                    "The API key is invalid, please try authenticating again."
                )
                raise SystemError(msg)
            if introspect_result is not None:
                return ServiceAccountProfile(
                    id=IdHelper.to_string(introspect_result.id),
                    name=introspect_result.name,
                    description=introspect_result.description,
                    created_by=introspect_result.created_by,
                    is_active=introspect_result.active,
                    obscured_key=f"****{token[-4:]}",
                )

    msg = (
        "No user profile or service account configured. To authenticate "
        "using an API key, run `tecton.login(tecton_url=<tecton_url>, tecton_api_key=<key>)`. "
        "To authenticate as your user, run `tecton login` with the CLI or "
        "`tecton.login(tecton_url=<tecton_url>)` in your notebook."
    )
    raise SystemError(msg)


def _check_user_can_read_org(user_id: str) -> bool:
    request = GetIsAuthorizedRequest(
        principal_type=PrincipalType.PRINCIPAL_TYPE_USER,
        principal_id=user_id,
        permissions=[
            Permission(
                resource_type=ResourceType.RESOURCE_TYPE_ORGANIZATION,
                action="read_organization",
            )
        ],
    )
    try:
        response = metadata_service.instance().GetIsAuthorized(request)
    except PermissionError as e:
        # if the user doesn't exist in SpiceDB, it will say resource not found
        return False
    return len(response.permissions) > 0 and response.permissions[0].action == "read_organization"


def logout() -> None:
    """
    Clear credentials from the interactive Python session that were previously set via `login()`.
    """
    _clear_credentials_impl()
    global notebook_login_temp_result
    notebook_login_temp_result = None
    printer.safe_print(
        "✅ Successfully cleared Service Account API key and User credentials from session. Use `tecton.login()` to re-authenticate."
    )
    if conf.get_or_none("TECTON_API_KEY") is not None:
        print(
            "👉️ TECTON_API_KEY is still set in an environment variable or in a Secret Manager, so that credential will still be used until it is cleared."
        )


def login(
    tecton_url: str,
    interactive: bool = True,
    tecton_api_key: Optional[str] = None,
) -> None:
    """
    Authenticate to a Tecton instance either as a user or with a Tecton API key.

    These credentials last for the duration of the interactive Python session. If
    both Service Account API key credentials and user credentials are set in the same
    session with multiple invocations of this method, user credentials will take priority
    when authenticating.

    If logging in as a user, the method will prompt you to open a link in your browser.
    Sign in to Tecton and it will display an authorization code to copy and paste into
    standard input back in your notebook.

    If your interactive environment does not support standard input (for example EMR
    notebooks do not), you can set `interactive=False` and use the 2-step login
    flow. First, run:
    ```
    tecton.login(tecton_url="https://<example>.tecton.ai", interactive=False)
    ```
    Then follow the URL to get your authorization code, and finally, run:
    ```
    tecton.complete_login("<authorization_code>")
    ```
    #### Example usages:
    To authenticate interactively as a user:
    ```
    tecton.login(tecton_url="https://<example>.tecton.ai")
    ```
    To authenticate non-interactively as a user
    ```
    tecton.login(tecton_url="https://<example>.tecton.ai", interactive=False)
    ```
    To authenticate with a Tecton API key
    ```
    tecton.login(tecton_url="https://<example>.tecton.ai", tecton_api_key="<api_key>")
    ```

    :param tecton_url: HTTP URL of the Tecton instance, e.g. `https://<example>.tecton.ai`
    :param interactive: Only for user login. When set to True, you will be prompted to pass the
                        authorization code via standard input to complete login. This relies on
                        your interactive environment supporting Python's builtin
                        [`input()`](https://docs.python.org/3/library/functions.html#input)
                        function. If your environment does not support this, you can call `login()`
                        with `interactive=False` to bypass the standard input prompt, and then pass
                        the authorization code to `tecton.complete_login("<authorization_code>")`
                        to complete login.
    :param tecton_api_key: Optional Tecton API key. Set to authenticate as a Service Account.
    """
    # Validate inputs
    if tecton_api_key:
        if not interactive:
            printer.safe_print(
                "If using `tecton_api_key`, cannot use the `interactive` parameter. Please use "
                '`tecton.login(tecton_url="https://<example>.tecton.ai", tecton_api_key="<api_key>")`.'
            )
            return

    url = _lint_tecton_url(tecton_url)

    if tecton_api_key:
        conf.delete_okta_tokens()
        tecton_api_service_url = urllib.parse.urljoin(url, "api")
        _set_credentials_impl(tecton_api_service_url=tecton_api_service_url, tecton_api_key=tecton_api_key)
        print("✅ Successfully set credentials.")
    else:
        _user_login(tecton_url, interactive)


def _user_login(url: str, interactive: bool) -> None:
    current_url = cluster_url()
    if current_url:
        if current_url == url:
            # logging into same cluster
            user_profile = None
            try:
                user_profile = okta.get_user_profile()
            except PermissionError as e:
                # Okta token expired
                print("Re-authenticating. Previous credentials no longer valid.")

            if user_profile:
                # test that the user can authenticate to the cluster
                if _check_user_can_read_org(user_profile.id):
                    print(
                        f"Already logged in to {current_url} as {user_profile}. To switch users, run `tecton.logout` then `tecton.login`"
                    )
                    return
                else:
                    print("Re-authenticating. Previous credentials no longer valid.")
        else:
            print(f"Re-authenticating. Switching from {current_url} to {url}")

    if interactive:
        try:
            # use BROWSER_MANUAL for now, can always make it nicer later
            temp_result, auth_code = _initiate_login_and_get_auth_code(url, okta.AuthFlowType.BROWSER_MANUAL)
        except EOFError:
            msg = "This environment does not support interactive login. Please use `tecton.login(tecton_url=<url>, interactive=False)`"
            raise SystemExit(msg)
        _complete_login_with_auth_code(temp_result, auth_code)
        printer.safe_print("✅ Authentication successful!")
    else:
        global notebook_login_temp_result
        notebook_login_temp_result = _initiate_noninterative_login(url)


def complete_login(authorization_code: str) -> None:
    """
    Complete non-interactive authentication to a Tecton instance as a user, by exchanging the authorization code for an access token.

    This requires that an authorization code was already first requested by calling
    `tecton.login(tecton_url="https://<example>.tecton.ai", interactive=False)`.
    Please see `login()` documentation for more details and example usage.

    :param authorization_code: Authorization code, copied from opening the URL printed after running
                               `tecton.login(tecton_url="https://<example>.tecton.ai", interactive=False)`
    """
    global notebook_login_temp_result
    if not notebook_login_temp_result:
        print(
            'Please generate an authorization code using `tecton.login(tecton_url="https://<example>.tecton.ai", interactive=False)` first.'
        )
        raise SystemExit()

    _complete_login_with_auth_code(
        temp_login_state=notebook_login_temp_result,
        auth_code=authorization_code,
    )

    printer.safe_print("✅ Authentication successful!")
    notebook_login_temp_result = None


def _initiate_noninterative_login(host: str) -> LoginTempState:
    """
    Used by Notebook SDK `tecton.login(interactive=False)` to request an auth code, but does not
    interactively get it. Does not return the auth code because the user will manually pass it later

    :param host:
    :return: LoginTempState with some login metadata that will be used to complete login in _complete_login_with_auth_code
    """
    host = _lint_tecton_url(host)
    login_configs = _get_login_configs(host)
    cli_client_id = login_configs["OKTA_CLI_CLIENT_ID"]

    flow = okta.OktaAuthorizationFlow(auth_flow_type=okta.AuthFlowType.NOTEBOOK_2_STEP)
    code_verifier, authorize_url, redirect_uri, _ = flow.generate_pkce_and_login_link(cli_client_id)

    print("Please visit the following link to login and access the authentication code:")
    print(f"{authorize_url}")
    print()
    print("Then use `tecton.complete_login(<authentication_code>)` to complete login")

    return LoginTempState(
        login_configs=login_configs,
        code_verifier=code_verifier,
        redirect_uri=redirect_uri,
        tecton_url=host,
        okta_auth_flow=flow,
    )


def _initiate_login_and_get_auth_code(
    host: str,
    auth_flow_type: okta.AuthFlowType,
    okta_session_token: Optional[str] = None,
) -> Tuple[LoginTempState, str]:
    """
    Used by both CLI `tecton login` and Notebook SDK `tecton.login(interactive=True)` to request an
    auth code and also get it.

    :param url: URL of Tecton deployment, e.g. https://staging.tecton.ai
    :param auth_flow_type: okta.AuthFlowType (browser, manual, or session token)
    :param okta_session_token: Optional string for auth_flow_type SESSION_TOKEN

    :return: A Tuple with:
             - LoginTempState with some login metadata that will be used to complete login in _complete_login_with_auth_code
             - str: the authorization code
    """
    host = _lint_tecton_url(host)
    login_configs = _get_login_configs(host)
    cli_client_id = login_configs["OKTA_CLI_CLIENT_ID"]

    flow = okta.OktaAuthorizationFlow(auth_flow_type=auth_flow_type, okta_session_token=okta_session_token)
    auth_code, code_verifier, redirect_uri = flow.get_authorization_code(cli_client_id)

    login_temp_state = LoginTempState(
        login_configs=login_configs,
        code_verifier=code_verifier,
        redirect_uri=redirect_uri,
        tecton_url=host,
        okta_auth_flow=flow,
    )
    return login_temp_state, auth_code


def _complete_login_with_auth_code(
    temp_login_state: LoginTempState,
    auth_code: str,
) -> None:
    """
    Completes login by exchanging the auth code for the oauth access token, and then
    setting tecton confs locally.
    """
    login_configs = temp_login_state.login_configs
    code_verifier = temp_login_state.code_verifier
    redirect_uri = temp_login_state.redirect_uri
    host = temp_login_state.tecton_url
    flow = temp_login_state.okta_auth_flow
    cli_client_id = login_configs["OKTA_CLI_CLIENT_ID"]

    access_token, _, refresh_token, access_token_expiration = flow.get_tokens(
        auth_code, code_verifier, redirect_uri, cli_client_id
    )
    if not access_token:
        printer.safe_print("Unable to obtain Tecton credentials")
        sys.exit(1)

    # when switching clusters
    if conf.get_or_none("API_SERVICE") != urllib.parse.urljoin(host, "api"):
        if conf.save_tecton_configs_enabled:
            # in the CLI, switch current workspace to prod
            switch_to_workspace(workspace.PROD_WORKSPACE_NAME)
        else:
            # in notebooks, clear MDS conf values from previous cluster
            conf._clear_metadata_server_config()
            # metadata service needs to be re-initialized, which will trigger new server config retrieval
            metadata_service.close_instance()

    new_api_service = urllib.parse.urljoin(host, "api")
    conf.set_login_configs(
        cli_client_id=cli_client_id,
        api_service=new_api_service,
        feature_service=new_api_service,
    )

    conf.set_okta_tokens(access_token, access_token_expiration, refresh_token)


def _lint_tecton_url(host: str) -> str:
    try:
        urllib.parse.urlparse(host)
    except Exception:
        printer.safe_print("Tecton Cluster URL must be a valid URL")
        sys.exit(1)
        # add this check for now since it can be hard to debug if you don't specify https and API_SERVICE fails
    if host is None or not (host.startswith(("https://", "http://localhost:"))):
        if host is not None and "//" not in host:
            return f"https://{host}"
        else:
            printer.safe_print("Tecton Cluster URL must start with https://")
            sys.exit(1)
    return host


def _get_login_configs(host: str) -> Dict[str, Any]:
    """
    Returns login configs from auth server
    """
    login_configs_url = urllib.parse.urljoin(host, "api/v1/metadata-service/get-login-configs")
    try:
        response = http.session().post(login_configs_url)
        response.raise_for_status()
    except requests.RequestException as e:
        raise SystemExit(e)
    configs = response.json()["key_values"]

    return configs



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/service_account__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.auditlog import metadata__client_pb2 as tecton__proto_dot_auditlog_dot_metadata____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n/tecton_proto/data/service_account__client.proto\x12\x11tecton_proto.data\x1a,tecton_proto/auditlog/metadata__client.proto\"\x98\x01\n\x0fNewClientSecret\x12$\n\tsecret_id\x18\x01 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x08secretId\x12&\n\ncreated_at\x18\x02 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\tcreatedAt\x12\x1f\n\x06status\x18\x03 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x06status\x12\x16\n\x06secret\x18\x04 \x01(\tR\x06secret\"\xd0\x01\n\x12MaskedClientSecret\x12$\n\tsecret_id\x18\x01 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x08secretId\x12&\n\ncreated_at\x18\x02 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\tcreatedAt\x12&\n\nupdated_at\x18\x03 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\tupdatedAt\x12\x1f\n\x06status\x18\x04 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x06status\x12#\n\rmasked_secret\x18\x05 \x01(\tR\x0cmaskedSecret*\xbe\x01\n\x1dServiceAccountCredentialsType\x12\x30\n,SERVICE_ACCOUNT_CREDENTIALS_TYPE_UNSPECIFIED\x10\x00\x12,\n(SERVICE_ACCOUNT_CREDENTIALS_TYPE_API_KEY\x10\x01\x12=\n9SERVICE_ACCOUNT_CREDENTIALS_TYPE_OAUTH_CLIENT_CREDENTIALS\x10\x02\x42;\n\x0f\x63om.tecton.dataP\x01Z&github.com/tecton-ai/tecton_proto/data')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.service_account__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001Z&github.com/tecton-ai/tecton_proto/data'
  _NEWCLIENTSECRET.fields_by_name['secret_id']._options = None
  _NEWCLIENTSECRET.fields_by_name['secret_id']._serialized_options = b'\352\231\275F\002\010\002'
  _NEWCLIENTSECRET.fields_by_name['created_at']._options = None
  _NEWCLIENTSECRET.fields_by_name['created_at']._serialized_options = b'\352\231\275F\002\010\002'
  _NEWCLIENTSECRET.fields_by_name['status']._options = None
  _NEWCLIENTSECRET.fields_by_name['status']._serialized_options = b'\352\231\275F\002\010\002'
  _MASKEDCLIENTSECRET.fields_by_name['secret_id']._options = None
  _MASKEDCLIENTSECRET.fields_by_name['secret_id']._serialized_options = b'\352\231\275F\002\010\002'
  _MASKEDCLIENTSECRET.fields_by_name['created_at']._options = None
  _MASKEDCLIENTSECRET.fields_by_name['created_at']._serialized_options = b'\352\231\275F\002\010\002'
  _MASKEDCLIENTSECRET.fields_by_name['updated_at']._options = None
  _MASKEDCLIENTSECRET.fields_by_name['updated_at']._serialized_options = b'\352\231\275F\002\010\002'
  _MASKEDCLIENTSECRET.fields_by_name['status']._options = None
  _MASKEDCLIENTSECRET.fields_by_name['status']._serialized_options = b'\352\231\275F\002\010\002'
  _SERVICEACCOUNTCREDENTIALSTYPE._serialized_start=483
  _SERVICEACCOUNTCREDENTIALSTYPE._serialized_end=673
  _NEWCLIENTSECRET._serialized_start=117
  _NEWCLIENTSECRET._serialized_end=269
  _MASKEDCLIENTSECRET._serialized_start=272
  _MASKEDCLIENTSECRET._serialized_end=480
# @@protoc_insertion_point(module_scope)



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from tecton_proto.common import id__client_pb2 as _id__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor
SERVING_STATE_DISABLED: ServingState
SERVING_STATE_ERROR: ServingState
SERVING_STATE_NOT_ENOUGH_DATA: ServingState
SERVING_STATE_OK: ServingState
SERVING_STATE_PENDING: ServingState
SERVING_STATE_RUNNING: ServingState
SERVING_STATE_UNSPECIFIED: ServingState

class FeatureViewMaterializationRanges(_message.Message):
    __slots__ = ["batch_materialization_ranges", "feature_view_id"]
    BATCH_MATERIALIZATION_RANGES_FIELD_NUMBER: _ClassVar[int]
    FEATURE_VIEW_ID_FIELD_NUMBER: _ClassVar[int]
    batch_materialization_ranges: _containers.RepeatedCompositeFieldContainer[StatusRange]
    feature_view_id: _id__client_pb2.Id
    def __init__(self, feature_view_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., batch_materialization_ranges: _Optional[_Iterable[_Union[StatusRange, _Mapping]]] = ...) -> None: ...

class FeatureViewServingStatusSummary(_message.Message):
    __slots__ = ["feature_view_id", "serving_status_summary"]
    FEATURE_VIEW_ID_FIELD_NUMBER: _ClassVar[int]
    SERVING_STATUS_SUMMARY_FIELD_NUMBER: _ClassVar[int]
    feature_view_id: _id__client_pb2.Id
    serving_status_summary: ServingStatusSummary
    def __init__(self, feature_view_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., serving_status_summary: _Optional[_Union[ServingStatusSummary, _Mapping]] = ...) -> None: ...

class FullFeatureServiceServingSummary(_message.Message):
    __slots__ = ["feature_service_serving_status_summary", "feature_view_serving_status_summaries"]
    FEATURE_SERVICE_SERVING_STATUS_SUMMARY_FIELD_NUMBER: _ClassVar[int]
    FEATURE_VIEW_SERVING_STATUS_SUMMARIES_FIELD_NUMBER: _ClassVar[int]
    feature_service_serving_status_summary: ServingStatusSummary
    feature_view_serving_status_summaries: _containers.RepeatedCompositeFieldContainer[FeatureViewServingStatusSummary]
    def __init__(self, feature_service_serving_status_summary: _Optional[_Union[ServingStatusSummary, _Mapping]] = ..., feature_view_serving_status_summaries: _Optional[_Iterable[_Union[FeatureViewServingStatusSummary, _Mapping]]] = ...) -> None: ...

class ServingStatus(_message.Message):
    __slots__ = ["errors", "serving_state"]
    ERRORS_FIELD_NUMBER: _ClassVar[int]
    SERVING_STATE_FIELD_NUMBER: _ClassVar[int]
    errors: _containers.RepeatedScalarFieldContainer[str]
    serving_state: ServingState
    def __init__(self, serving_state: _Optional[_Union[ServingState, str]] = ..., errors: _Optional[_Iterable[str]] = ...) -> None: ...

class ServingStatusSummary(_message.Message):
    __slots__ = ["batch_materialization_ranges", "most_recent_batch_processing_time", "most_recent_ready_time", "offline_readiness_ranges", "online_readiness_ranges", "streaming_serving_status", "streaming_start_time"]
    BATCH_MATERIALIZATION_RANGES_FIELD_NUMBER: _ClassVar[int]
    MOST_RECENT_BATCH_PROCESSING_TIME_FIELD_NUMBER: _ClassVar[int]
    MOST_RECENT_READY_TIME_FIELD_NUMBER: _ClassVar[int]
    OFFLINE_READINESS_RANGES_FIELD_NUMBER: _ClassVar[int]
    ONLINE_READINESS_RANGES_FIELD_NUMBER: _ClassVar[int]
    STREAMING_SERVING_STATUS_FIELD_NUMBER: _ClassVar[int]
    STREAMING_START_TIME_FIELD_NUMBER: _ClassVar[int]
    batch_materialization_ranges: _containers.RepeatedCompositeFieldContainer[FeatureViewMaterializationRanges]
    most_recent_batch_processing_time: _timestamp_pb2.Timestamp
    most_recent_ready_time: _timestamp_pb2.Timestamp
    offline_readiness_ranges: _containers.RepeatedCompositeFieldContainer[StatusRange]
    online_readiness_ranges: _containers.RepeatedCompositeFieldContainer[StatusRange]
    streaming_serving_status: ServingStatus
    streaming_start_time: _timestamp_pb2.Timestamp
    def __init__(self, offline_readiness_ranges: _Optional[_Iterable[_Union[StatusRange, _Mapping]]] = ..., online_readiness_ranges: _Optional[_Iterable[_Union[StatusRange, _Mapping]]] = ..., batch_materialization_ranges: _Optional[_Iterable[_Union[FeatureViewMaterializationRanges, _Mapping]]] = ..., streaming_start_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., streaming_serving_status: _Optional[_Union[ServingStatus, _Mapping]] = ..., most_recent_ready_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., most_recent_batch_processing_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class StatusRange(_message.Message):
    __slots__ = ["begin_inclusive", "end_exclusive", "status"]
    BEGIN_INCLUSIVE_FIELD_NUMBER: _ClassVar[int]
    END_EXCLUSIVE_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    begin_inclusive: _timestamp_pb2.Timestamp
    end_exclusive: _timestamp_pb2.Timestamp
    status: ServingStatus
    def __init__(self, begin_inclusive: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., end_exclusive: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., status: _Optional[_Union[ServingStatus, _Mapping]] = ...) -> None: ...

class ServingState(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from abc import ABC
from abc import abstractmethod
from types import MappingProxyType
from typing import Mapping
from typing import NamedTuple

from tecton_core.specs.secret_spec import SecretSpec
from tecton_proto.common.secret__client_pb2 import SecretReference


class SecretCacheKey(NamedTuple):
    scope: str
    key: str


class SecretResolver(ABC):
    """Abstract Secret Resolver to resolve secret references to their values.

    Abstract class is used to allow for different implementations of secret resolution, for example,
    fetching secrets from MDS during materialization, or fetching secrets from a environment in local development.
    """

    @abstractmethod
    def resolve(self, secret: SecretReference) -> str:
        """Resolve the secret reference to its value."""
        raise NotImplementedError

    def resolve_map(self, secrets: Mapping[str, SecretReference]) -> Mapping:
        """Resolve a map of secret references to their values."""
        resolved_secrets_dict = {name: self.resolve(secret) for name, secret in secrets.items()}
        return MappingProxyType(resolved_secrets_dict)

    def resolve_spec_map(self, secrets: Mapping[str, SecretSpec]) -> Mapping:
        map = {
            name: SecretReference(scope=secret.scope, key=secret.key, is_local=secret.is_local)
            for (name, secret) in secrets.items()
        }
        return self.resolve_map(map)



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from tecton_proto.common import id__client_pb2 as _id__client_pb2
from tecton_proto.common import schema__client_pb2 as _schema__client_pb2
from tecton_proto.common import spark_schema__client_pb2 as _spark_schema__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor
LOGGED: SavedFeatureDataFrameType
NOT_SET: SavedFeatureDataFrameType
SAVED: SavedFeatureDataFrameType
SAVED_FEATURED_DATAFRAME_FORMAT_LEGACY: SavedFeatureDataFrameFormat
SAVED_FEATURED_DATAFRAME_FORMAT_UNIFIED_STORAGE: SavedFeatureDataFrameFormat
SAVED_FEATURED_DATAFRAME_FORMAT_UNSPECIFIED: SavedFeatureDataFrameFormat

class LoggedDataset(_message.Message):
    __slots__ = ["join_key_column_names", "timestamp_column_name"]
    JOIN_KEY_COLUMN_NAMES_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_COLUMN_NAME_FIELD_NUMBER: _ClassVar[int]
    join_key_column_names: _containers.RepeatedScalarFieldContainer[str]
    timestamp_column_name: str
    def __init__(self, join_key_column_names: _Optional[_Iterable[str]] = ..., timestamp_column_name: _Optional[str] = ...) -> None: ...

class SavedDataset(_message.Message):
    __slots__ = ["creation_task_id", "event_column_names", "timestamp_column_name"]
    CREATION_TASK_ID_FIELD_NUMBER: _ClassVar[int]
    EVENT_COLUMN_NAMES_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_COLUMN_NAME_FIELD_NUMBER: _ClassVar[int]
    creation_task_id: _id__client_pb2.Id
    event_column_names: _containers.RepeatedScalarFieldContainer[str]
    timestamp_column_name: str
    def __init__(self, creation_task_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., event_column_names: _Optional[_Iterable[str]] = ..., timestamp_column_name: _Optional[str] = ...) -> None: ...

class SavedFeatureDataFrame(_message.Message):
    __slots__ = ["dataframe_location", "feature_package_id", "feature_package_name", "feature_service_id", "feature_service_name", "format", "info", "join_key_column_names", "logged_dataset", "saved_dataset", "saved_feature_dataframe_id", "schema", "state_update_entry_commit_id", "timestamp_column_name", "type", "unified_schema"]
    DATAFRAME_LOCATION_FIELD_NUMBER: _ClassVar[int]
    FEATURE_PACKAGE_ID_FIELD_NUMBER: _ClassVar[int]
    FEATURE_PACKAGE_NAME_FIELD_NUMBER: _ClassVar[int]
    FEATURE_SERVICE_ID_FIELD_NUMBER: _ClassVar[int]
    FEATURE_SERVICE_NAME_FIELD_NUMBER: _ClassVar[int]
    FORMAT_FIELD_NUMBER: _ClassVar[int]
    INFO_FIELD_NUMBER: _ClassVar[int]
    JOIN_KEY_COLUMN_NAMES_FIELD_NUMBER: _ClassVar[int]
    LOGGED_DATASET_FIELD_NUMBER: _ClassVar[int]
    SAVED_DATASET_FIELD_NUMBER: _ClassVar[int]
    SAVED_FEATURE_DATAFRAME_ID_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_FIELD_NUMBER: _ClassVar[int]
    STATE_UPDATE_ENTRY_COMMIT_ID_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_COLUMN_NAME_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    UNIFIED_SCHEMA_FIELD_NUMBER: _ClassVar[int]
    dataframe_location: str
    feature_package_id: _id__client_pb2.Id
    feature_package_name: str
    feature_service_id: _id__client_pb2.Id
    feature_service_name: str
    format: SavedFeatureDataFrameFormat
    info: SavedFeatureDataFrameInfo
    join_key_column_names: _containers.RepeatedScalarFieldContainer[str]
    logged_dataset: LoggedDataset
    saved_dataset: SavedDataset
    saved_feature_dataframe_id: _id__client_pb2.Id
    schema: _spark_schema__client_pb2.SparkSchema
    state_update_entry_commit_id: str
    timestamp_column_name: str
    type: SavedFeatureDataFrameType
    unified_schema: _schema__client_pb2.Schema
    def __init__(self, saved_feature_dataframe_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., info: _Optional[_Union[SavedFeatureDataFrameInfo, _Mapping]] = ..., dataframe_location: _Optional[str] = ..., feature_package_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., feature_service_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., feature_package_name: _Optional[str] = ..., feature_service_name: _Optional[str] = ..., state_update_entry_commit_id: _Optional[str] = ..., join_key_column_names: _Optional[_Iterable[str]] = ..., timestamp_column_name: _Optional[str] = ..., schema: _Optional[_Union[_spark_schema__client_pb2.SparkSchema, _Mapping]] = ..., type: _Optional[_Union[SavedFeatureDataFrameType, str]] = ..., format: _Optional[_Union[SavedFeatureDataFrameFormat, str]] = ..., unified_schema: _Optional[_Union[_schema__client_pb2.Schema, _Mapping]] = ..., saved_dataset: _Optional[_Union[SavedDataset, _Mapping]] = ..., logged_dataset: _Optional[_Union[LoggedDataset, _Mapping]] = ...) -> None: ...

class SavedFeatureDataFrameInfo(_message.Message):
    __slots__ = ["created_at", "data_deleted_at", "is_archived", "is_data_deleted", "name", "workspace"]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    DATA_DELETED_AT_FIELD_NUMBER: _ClassVar[int]
    IS_ARCHIVED_FIELD_NUMBER: _ClassVar[int]
    IS_DATA_DELETED_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    WORKSPACE_FIELD_NUMBER: _ClassVar[int]
    created_at: _timestamp_pb2.Timestamp
    data_deleted_at: _timestamp_pb2.Timestamp
    is_archived: bool
    is_data_deleted: bool
    name: str
    workspace: str
    def __init__(self, name: _Optional[str] = ..., created_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., is_archived: bool = ..., workspace: _Optional[str] = ..., is_data_deleted: bool = ..., data_deleted_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class SavedFeatureDataFrameType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class SavedFeatureDataFrameFormat(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/server_group__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.args import basic_info__client_pb2 as tecton__proto_dot_args_dot_basic__info____client__pb2
from tecton_proto.args import diff_options__client_pb2 as tecton__proto_dot_args_dot_diff__options____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.common import scaling_config__client_pb2 as tecton__proto_dot_common_dot_scaling__config____client__pb2
from tecton_proto.common import server_group_type__client_pb2 as tecton__proto_dot_common_dot_server__group__type____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,tecton_proto/args/server_group__client.proto\x12\x11tecton_proto.args\x1a*tecton_proto/args/basic_info__client.proto\x1a,tecton_proto/args/diff_options__client.proto\x1a$tecton_proto/common/id__client.proto\x1a\x30tecton_proto/common/scaling_config__client.proto\x1a\x33tecton_proto/common/server_group_type__client.proto\"\x8c\x07\n\x0fServerGroupArgs\x12?\n\x0fserver_group_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\rserverGroupId\x12\x37\n\x04info\x18\x02 \x01(\x0b\x32\x1c.tecton_proto.args.BasicInfoB\x05\x92M\x02\x10\x01R\x04info\x12P\n\x11server_group_type\x18\x03 \x01(\x0e\x32$.tecton_proto.common.ServerGroupTypeR\x0fserverGroupType\x12s\n\x1btransform_server_group_args\x18\x08 \x01(\x0b\x32+.tecton_proto.args.TransformServerGroupArgsB\x05\x92M\x02\x10\x01H\x00R\x18transformServerGroupArgs\x12m\n\x19\x66\x65\x61ture_server_group_args\x18\x0b \x01(\x0b\x32).tecton_proto.args.FeatureServerGroupArgsB\x05\x92M\x02\x10\x01H\x00R\x16\x66\x65\x61tureServerGroupArgs\x12.\n\x0fprevent_destroy\x18\t \x01(\x08\x42\x05\x92M\x02\x08\x01R\x0epreventDestroy\x12P\n\x07options\x18\n \x03(\x0b\x32/.tecton_proto.args.ServerGroupArgs.OptionsEntryB\x05\x92M\x02\x08\x01R\x07options\x12^\n\x12\x61utoscaling_config\x18\x0c \x01(\x0b\x32&.tecton_proto.common.AutoscalingConfigB\x05\x92M\x02\x08\x01H\x01R\x11\x61utoscalingConfig\x12t\n\x1aprovisioned_scaling_config\x18\r \x01(\x0b\x32-.tecton_proto.common.ProvisionedScalingConfigB\x05\x92M\x02\x08\x01H\x01R\x18provisionedScalingConfig\x1a:\n\x0cOptionsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\x0b\n\ttype_argsB\x10\n\x0escaling_configJ\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06J\x04\x08\x06\x10\x07J\x04\x08\x07\x10\x08\"<\n\x18TransformServerGroupArgs\x12 \n\x0b\x65nvironment\x18\x01 \x01(\tR\x0b\x65nvironment\"\x18\n\x16\x46\x65\x61tureServerGroupArgs\"k\n\x14ServerGroupReference\x12?\n\x0fserver_group_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\rserverGroupId\x12\x12\n\x04name\x18\x02 \x01(\tR\x04nameB;\n\x0f\x63om.tecton.argsP\x01Z&github.com/tecton-ai/tecton_proto/args')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.args.server_group__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.argsP\001Z&github.com/tecton-ai/tecton_proto/args'
  _SERVERGROUPARGS_OPTIONSENTRY._options = None
  _SERVERGROUPARGS_OPTIONSENTRY._serialized_options = b'8\001'
  _SERVERGROUPARGS.fields_by_name['info']._options = None
  _SERVERGROUPARGS.fields_by_name['info']._serialized_options = b'\222M\002\020\001'
  _SERVERGROUPARGS.fields_by_name['transform_server_group_args']._options = None
  _SERVERGROUPARGS.fields_by_name['transform_server_group_args']._serialized_options = b'\222M\002\020\001'
  _SERVERGROUPARGS.fields_by_name['feature_server_group_args']._options = None
  _SERVERGROUPARGS.fields_by_name['feature_server_group_args']._serialized_options = b'\222M\002\020\001'
  _SERVERGROUPARGS.fields_by_name['prevent_destroy']._options = None
  _SERVERGROUPARGS.fields_by_name['prevent_destroy']._serialized_options = b'\222M\002\010\001'
  _SERVERGROUPARGS.fields_by_name['options']._options = None
  _SERVERGROUPARGS.fields_by_name['options']._serialized_options = b'\222M\002\010\001'
  _SERVERGROUPARGS.fields_by_name['autoscaling_config']._options = None
  _SERVERGROUPARGS.fields_by_name['autoscaling_config']._serialized_options = b'\222M\002\010\001'
  _SERVERGROUPARGS.fields_by_name['provisioned_scaling_config']._options = None
  _SERVERGROUPARGS.fields_by_name['provisioned_scaling_config']._serialized_options = b'\222M\002\010\001'
  _SERVERGROUPARGS._serialized_start=299
  _SERVERGROUPARGS._serialized_end=1207
  _SERVERGROUPARGS_OPTIONSENTRY._serialized_start=1094
  _SERVERGROUPARGS_OPTIONSENTRY._serialized_end=1152
  _TRANSFORMSERVERGROUPARGS._serialized_start=1209
  _TRANSFORMSERVERGROUPARGS._serialized_end=1269
  _FEATURESERVERGROUPARGS._serialized_start=1271
  _FEATURESERVERGROUPARGS._serialized_end=1295
  _SERVERGROUPREFERENCE._serialized_start=1297
  _SERVERGROUPREFERENCE._serialized_end=1404
# @@protoc_insertion_point(module_scope)



from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class CommandsExecuteRequest(_message.Message):
    __slots__ = ["clusterId", "command", "contextId", "language"]
    CLUSTERID_FIELD_NUMBER: _ClassVar[int]
    COMMAND_FIELD_NUMBER: _ClassVar[int]
    CONTEXTID_FIELD_NUMBER: _ClassVar[int]
    LANGUAGE_FIELD_NUMBER: _ClassVar[int]
    clusterId: str
    command: str
    contextId: str
    language: str
    def __init__(self, language: _Optional[str] = ..., clusterId: _Optional[str] = ..., contextId: _Optional[str] = ..., command: _Optional[str] = ...) -> None: ...

class CommandsExecuteResponse(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class CommandsStatusRequest(_message.Message):
    __slots__ = ["clusterId", "commandId", "contextId"]
    CLUSTERID_FIELD_NUMBER: _ClassVar[int]
    COMMANDID_FIELD_NUMBER: _ClassVar[int]
    CONTEXTID_FIELD_NUMBER: _ClassVar[int]
    clusterId: str
    commandId: str
    contextId: str
    def __init__(self, clusterId: _Optional[str] = ..., contextId: _Optional[str] = ..., commandId: _Optional[str] = ...) -> None: ...

class CommandsStatusResponse(_message.Message):
    __slots__ = ["results", "status"]
    RESULTS_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    results: Results
    status: str
    def __init__(self, results: _Optional[_Union[Results, _Mapping]] = ..., status: _Optional[str] = ...) -> None: ...

class ContextDestroyRequest(_message.Message):
    __slots__ = ["clusterId", "contextId"]
    CLUSTERID_FIELD_NUMBER: _ClassVar[int]
    CONTEXTID_FIELD_NUMBER: _ClassVar[int]
    clusterId: str
    contextId: str
    def __init__(self, clusterId: _Optional[str] = ..., contextId: _Optional[str] = ...) -> None: ...

class ContextDestroyResponse(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class ContextStatusRequest(_message.Message):
    __slots__ = ["clusterId", "contextId"]
    CLUSTERID_FIELD_NUMBER: _ClassVar[int]
    CONTEXTID_FIELD_NUMBER: _ClassVar[int]
    clusterId: str
    contextId: str
    def __init__(self, clusterId: _Optional[str] = ..., contextId: _Optional[str] = ...) -> None: ...

class ContextStatusResponse(_message.Message):
    __slots__ = ["error", "id", "status"]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    error: str
    id: str
    status: str
    def __init__(self, id: _Optional[str] = ..., status: _Optional[str] = ..., error: _Optional[str] = ...) -> None: ...

class ContextsCreateRequest(_message.Message):
    __slots__ = ["clusterId", "language"]
    CLUSTERID_FIELD_NUMBER: _ClassVar[int]
    LANGUAGE_FIELD_NUMBER: _ClassVar[int]
    clusterId: str
    language: str
    def __init__(self, language: _Optional[str] = ..., clusterId: _Optional[str] = ...) -> None: ...

class ContextsCreateResponse(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class Results(_message.Message):
    __slots__ = ["cause", "data", "resultType"]
    CAUSE_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    RESULTTYPE_FIELD_NUMBER: _ClassVar[int]
    cause: str
    data: str
    resultType: str
    def __init__(self, data: _Optional[str] = ..., resultType: _Optional[str] = ..., cause: _Optional[str] = ...) -> None: ...



import click

from tecton import __version__
from tecton.cli.command import TectonCommand
from tecton.cli.printer import safe_print


@click.command(cls=TectonCommand)
def upgrade():
    safe_print(f"No code changes are necessary to upgrade to Tecton SDK version {__version__}.")



from __future__ import annotations

import importlib
from dataclasses import dataclass
from pathlib import Path
from types import MappingProxyType
from typing import Any
from typing import Iterator
from typing import Mapping
from typing import MutableMapping
from typing import Optional

import numpy
import pyarrow

from tecton_core import data_types


_LOAD_FN_NAME = "load_context"
_RESERVED_MODEL_CONTEXT_KEY = "model"
_TECTON_MODEL_SPEC_NAME = "_tecton_custom_model"

_CONTEXT_PARAM = Mapping[str, Any]
_DEFAULT_BATCH_SIZE = 50_000


def tecton_type_to_pyarrow(tecton_type: data_types.DataType) -> pyarrow.DataType:
    if tecton_type == data_types.Int32Type():
        return pyarrow.int32()
    if tecton_type == data_types.Int64Type():
        return pyarrow.int64()
    if tecton_type == data_types.Float32Type():
        return pyarrow.float32()
    if tecton_type == data_types.Float64Type():
        return pyarrow.float64()
    if tecton_type == data_types.StringType():
        return pyarrow.string()
    if isinstance(tecton_type, data_types.ArrayType):
        return pyarrow.list_(tecton_type_to_pyarrow(tecton_type.element_type))
    msg = f"Invalid tecton type found {tecton_type}"
    raise ValueError(msg)


def pyarrow_batch_to_numpy_dict(batch: pyarrow.RecordBatch) -> Mapping[str, numpy.ndarray]:
    # zero_copy_only=False is needed to allow conversion of non-primitive types see more:
    # https://arrow.apache.org/docs/python/generated/pyarrow.Int16Array.html#pyarrow.Int16Array.to_numpy
    return {name: column.to_numpy(zero_copy_only=False) for name, column in zip(batch.schema.names, batch.columns)}


class CustomModelContainerBase:
    _data_dir: Path
    _model_file_path: str

    # Late init params
    _context: MutableMapping[str, Any]

    def __init__(self, data_dir: str, model_file_path: str = "model.py") -> None:
        self._data_dir = Path(data_dir)
        self._model_file_path = model_file_path
        self._context = {}

    def load(self):
        model_module = self._load_model_module()
        _load_fn = getattr(model_module, _LOAD_FN_NAME, None)

        if not _load_fn:
            msg = f"`{_LOAD_FN_NAME}` function is missing in the model repo"
            raise AttributeError(msg)

        _load_fn(self._data_dir, self._context)
        if not self._model:
            msg = "No 'model' found in the context. `load` function should initialize the model and put it in the context with 'model' key."
            raise ValueError(msg)
        return model_module

    def predict(self, input: pyarrow.RecordBatch) -> numpy.ndarray:
        raise NotImplementedError

    def _load_model_module(self):
        spec = importlib.util.spec_from_file_location(_TECTON_MODEL_SPEC_NAME, self._data_dir / self._model_file_path)
        model_module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(model_module)
        return model_module

    @property
    def _model(self):
        model = self._context.get(_RESERVED_MODEL_CONTEXT_KEY, None)
        return model


# TODO(jiadong): Implement custom model batch function.
def default_batch_fn(
    batch: pyarrow.RecordBatch,
) -> Iterator[pyarrow.RecordBatch]:
    index = 0
    while index < batch.num_rows:
        num_rows = min(_DEFAULT_BATCH_SIZE, batch.num_rows - index)
        yield batch.take(list(range(index, index + num_rows)))
        index += num_rows


@dataclass
class CustomModelBatchFuncConfig:
    # TODO(jiadong): Add necessary params to batch function config such as user-provided batch size.

    # Note: _final_kwargs is a late init object.
    _final_kwargs: Optional[Mapping[str, Any]] = None

    @classmethod
    def create(cls) -> CustomModelBatchFuncConfig:
        return cls()

    def load(self):
        self._final_kwargs = MappingProxyType({})
        return self

    def kwargs(self) -> Mapping[str, Any]:
        if self._final_kwargs is None:
            msg = "`load` must be called prior to calling `kwargs`."
            raise ValueError(msg)
        return self._final_kwargs



from __future__ import annotations

from typing import List

from tecton_core import schema
from tecton_proto.modelartifactservice import model_artifact_data__client_pb2 as model_artifact_service_pb2


class ModelArtifactInfo:
    _proto: model_artifact_service_pb2.ModelArtifactInfo
    _model_type_enum = model_artifact_service_pb2.ModelType

    def __init__(self, proto: model_artifact_service_pb2.ModelArtifactInfo) -> None:
        self._proto = proto

    def __repr__(self):
        return f"ModelArtifactInfo(name={self.model_name}, id={self.model_artifact_id}, type={self.model_type})"

    @property
    def model_name(self) -> str:
        return self._proto.name

    @property
    def model_artifact_id(self) -> str:
        return self._proto.id

    @property
    def storage_path(self) -> str:
        return self._proto.storage_path

    @property
    def model_file_path(self) -> str:
        return self._proto.model_file_path

    @property
    def input_schema(self) -> List[schema.Column]:
        return [schema.Column(name, dtype) for name, dtype in schema.Schema(self._proto.input_schema).to_dict().items()]

    @property
    def model_type(self) -> str:
        return self._model_type_enum.Name(self._proto.type)



from datetime import datetime
from datetime import timedelta
from typing import Optional
from typing import Union

import tecton_core.tecton_pendulum as pendulum
from tecton_core import specs
from tecton_core import time_utils
from tecton_core.compute_mode import ComputeMode
from tecton_core.feature_definition_wrapper import FeatureDefinitionWrapper
from tecton_core.feature_set_config import FeatureDefinitionAndJoinConfig
from tecton_core.feature_set_config import FeatureSetConfig
from tecton_core.query import builder
from tecton_core.query import nodes
from tecton_core.query.dialect import Dialect
from tecton_core.query.node_interface import DataframeWrapper
from tecton_core.query.node_interface import NodeRef
from tecton_core.query.retrieval_params import GetFeaturesForEventsParams
from tecton_core.query.retrieval_params import GetFeaturesInRangeParams


def get_features_from_params(
    params: Union[GetFeaturesInRangeParams, GetFeaturesForEventsParams],
    spine: Optional[DataframeWrapper] = None,
    entities: Optional[DataframeWrapper] = None,
) -> NodeRef:
    # Spine is required for GetFeaturesForEventsParams, it needs to be in a DataframeWrapper
    if isinstance(params, GetFeaturesForEventsParams):
        if spine is None:
            error = "Spine is required for GetFeaturesForEventsParams"
            raise ValueError(error)
        if isinstance(params.fco, specs.FeatureServiceSpec):
            return get_features_for_events_for_feature_service_qt(
                dialect=params.compute_mode.default_dialect(),
                compute_mode=params.compute_mode,
                feature_set_config=params.feature_set_config,
                spine=spine,
                timestamp_key=params.timestamp_key,
                from_source=params.from_source,
            )
        elif isinstance(params.fco, FeatureDefinitionWrapper):
            return get_features_for_events_qt(
                dialect=params.compute_mode.default_dialect(),
                compute_mode=params.compute_mode,
                feature_definition=params.fco,
                spine=spine,
                timestamp_key=params.timestamp_key,
                from_source=params.from_source,
            )
    elif isinstance(params, GetFeaturesInRangeParams):
        return get_features_in_range_qt(
            dialect=params.compute_mode.default_dialect(),
            compute_mode=params.compute_mode,
            feature_definition=params.fco,
            start_time=params.start_time,
            end_time=params.end_time,
            max_lookback=params.max_lookback,
            entities=entities,
            from_source=params.from_source,
        )
    else:
        error = f"Unsupported params type: {type(params)}"
        raise ValueError(error)


def get_features_in_range_qt(
    dialect: Dialect,
    compute_mode: ComputeMode,
    feature_definition: FeatureDefinitionWrapper,
    start_time: Union[pendulum.DateTime, datetime],
    end_time: Union[pendulum.DateTime, datetime],
    max_lookback: Optional[timedelta],
    entities: Optional[DataframeWrapper],
    from_source: Optional[bool],
) -> NodeRef:
    start_time = pendulum.instance(start_time)
    if feature_definition.feature_start_timestamp is not None:
        start_time = max(start_time, feature_definition.feature_start_timestamp)
    query_time_range = pendulum.Period(start_time, pendulum.instance(end_time))

    if feature_definition.is_temporal or feature_definition.is_feature_table:
        lookback_time_range = time_utils.temporal_fv_get_feature_data_time_limits(
            feature_definition, query_time_range, max_lookback
        )
        qt = builder.build_temporal_time_range_validity_query(
            dialect=dialect,
            compute_mode=compute_mode,
            fd=feature_definition,
            from_source=from_source,
            query_time_range=query_time_range,
            lookback_time_range=lookback_time_range,
            entities=entities,
        )
    else:
        feature_data_time_limits = time_utils.get_feature_data_time_limits(
            fd=feature_definition,
            spine_time_limits=query_time_range,
        )
        qt = builder.build_aggregated_time_range_validity_query(
            dialect=dialect,
            compute_mode=compute_mode,
            fdw=feature_definition,
            feature_data_time_limits=feature_data_time_limits,
            query_time_range=query_time_range,
            from_source=from_source,
            entities=entities,
        )
    return qt


def get_features_for_events_for_feature_service_qt(
    dialect: Dialect,
    compute_mode: ComputeMode,
    feature_set_config: FeatureSetConfig,
    spine: DataframeWrapper,
    timestamp_key: Optional[str],
    from_source: Optional[bool],
) -> NodeRef:
    return _get_historical_features_for_feature_set(
        dialect, compute_mode, feature_set_config, spine, timestamp_key, from_source
    )


def _get_historical_features_for_feature_set(
    dialect: Dialect,
    compute_mode: ComputeMode,
    feature_set_config: FeatureSetConfig,
    spine: DataframeWrapper,
    timestamp_key: Optional[str],
    from_source: Optional[bool],
) -> NodeRef:
    user_data_node = nodes.UserSpecifiedDataNode(dialect, compute_mode, spine, {"timestamp_key": timestamp_key})
    user_data_node = nodes.ConvertTimestampToUTCNode(dialect, compute_mode, user_data_node.as_ref(), timestamp_key)

    tree = builder.build_feature_set_config_querytree(
        dialect,
        compute_mode,
        feature_set_config,
        user_data_node.as_ref(),
        timestamp_key,
        from_source,
    )
    return tree


def get_features_for_events_qt(
    dialect: Dialect,
    compute_mode: ComputeMode,
    feature_definition: FeatureDefinitionWrapper,
    spine: DataframeWrapper,
    timestamp_key: Optional[str],
    from_source: Optional[bool],
) -> NodeRef:
    return _point_in_time_get_historical_features_for_feature_definition(
        dialect, compute_mode, feature_definition, spine, timestamp_key, from_source
    )


def _point_in_time_get_historical_features_for_feature_definition(
    dialect: Dialect,
    compute_mode: ComputeMode,
    feature_definition: FeatureDefinitionWrapper,
    spine: DataframeWrapper,
    timestamp_key: Optional[str],
    from_source: Optional[bool],
) -> NodeRef:
    dac = FeatureDefinitionAndJoinConfig.from_feature_definition(feature_definition)
    user_data_node_metadata = {}
    user_data_node = nodes.UserSpecifiedDataNode(dialect, compute_mode, spine, user_data_node_metadata)

    if timestamp_key:
        user_data_node_metadata["timestamp_key"] = timestamp_key
        user_data_node = nodes.ConvertTimestampToUTCNode(dialect, compute_mode, user_data_node.as_ref(), timestamp_key)

    qt = builder.build_spine_join_querytree(
        dialect=dialect,
        compute_mode=compute_mode,
        dac=dac,
        spine_node=user_data_node.as_ref(),
        spine_time_field=timestamp_key,
        from_source=from_source,
        use_namespace_feature_prefix=True,
    )

    return qt



from typing import Dict
from typing import Optional
from typing import Union

import attrs
from typeguard import typechecked

from tecton._internals import sdk_decorators
from tecton._internals import validations_api
from tecton.framework import base_tecton_object
from tecton.framework import configs
from tecton_core import conf
from tecton_core import id_helper
from tecton_core import specs
from tecton_core.repo_file_handler import construct_fco_source_info
from tecton_proto.args import basic_info__client_pb2 as basic_info_pb2
from tecton_proto.args import fco_args__client_pb2 as fco_args_pb2
from tecton_proto.args import server_group__client_pb2 as server_group_pb2
from tecton_proto.common import server_group_type__client_pb2 as server_group_type_pb2
from tecton_proto.validation import validator__client_pb2 as validator_pb2


@attrs.define(eq=False)
class BaseServerGroup(base_tecton_object.BaseTectonObject):
    """Base class for server groups."""

    _args: Optional[server_group_pb2.ServerGroupArgs] = attrs.field(repr=False, on_setattr=attrs.setters.frozen)
    scaling_config: Optional[Union[configs.AutoscalingConfig, configs.ProvisionedScalingConfig]] = attrs.field(
        repr=False
    )
    _spec: Optional[specs.FeatureServerGroupSpec] = attrs.field(repr=False)

    @sdk_decorators.assert_local_object
    def _build_and_resolve_args(self, objects) -> fco_args_pb2.FcoArgs:
        return fco_args_pb2.FcoArgs(server_group=self._args)

    @classmethod
    @typechecked
    def _from_spec(cls, spec: specs.FeatureServerGroupSpec) -> "FeatureServerGroup":
        """Create a Feature Service from directly from a spec. Specs are assumed valid and will not be re-validated."""
        info = base_tecton_object.TectonObjectInfo.from_spec(spec)
        obj = cls.__new__(cls)
        obj.__attrs_init__(
            info=info,
            spec=spec,
            args=None,
            source_info=None,
            scaling_config=specs.scaling_config,
        )
        return obj

    def _build_fco_validation_args(self) -> validator_pb2.FcoValidationArgs:
        if self.info._is_local_object:
            return validator_pb2.FcoValidationArgs(
                server_group=validator_pb2.ServerGroupValidationArgs(
                    args=self._args,
                )
            )
        else:
            return self._spec.validation_args

    def _validate(self) -> None:
        validations_api.run_backend_validation_and_assert_valid(
            self,
            validator_pb2.ValidationRequest(
                validation_args=[self._build_fco_validation_args()],
            ),
        )


@attrs.define(eq=False)
class FeatureServerGroup(BaseServerGroup):
    """
    Configuration used to specify the options for provisioning a server group for the feature server.
    """

    def __init__(
        self,
        *,
        name: str,
        scaling_config: Optional[Union[configs.AutoscalingConfig, configs.ProvisionedScalingConfig]],
        description: Optional[str] = None,
        tags: Optional[Dict[str, str]] = None,
        owner: Optional[str] = None,
        prevent_destroy: bool = False,
        options: Optional[Dict[str, str]] = None,
    ):
        """
        Configuration used to specify the feature server group options.

        :param name: A unique name for the Feature Server Group.
        :param description: A human-readable description.
        :param tags: Tags associated with this Tecton Object (key-value pairs of arbitrary metadata).
        :param owner: Owner name (typically the email of the primary maintainer).
        :param prevent_destroy: If True, this Tecton object will be blocked from being deleted or re-created (i.e. a
            destructive update) during tecton plan/apply. To remove or update this object, `prevent_destroy` must be set to False
            via the same tecton apply or a separate tecton apply. `prevent_destroy` can be used to prevent accidental changes
            such as inadvertantly deleting a Feature Service used in production or recreating a Feature View that
            triggers expensive rematerialization jobs. `prevent_destroy` also blocks changes to dependent Tecton objects
            that would trigger a recreate of the tagged object, e.g. if `prevent_destroy` is set on a Feature Service,
            that will also prevent deletions or re-creates of Feature Views used in that service. `prevent_destroy` is
            only enforced in live (i.e. non-dev) workspaces.
        :param scaling_config: A configuration for creating an autoscaling or provisioned server group.
        :param options: Additional options to configure the Feature Server Group. Used for advanced use cases and beta features.
        """
        args = server_group_pb2.ServerGroupArgs(
            server_group_id=id_helper.IdHelper.generate_id(),
            info=basic_info_pb2.BasicInfo(name=name, description=description, tags=tags, owner=owner),
            prevent_destroy=prevent_destroy,
            feature_server_group_args=server_group_pb2.FeatureServerGroupArgs(),
            server_group_type=server_group_type_pb2.SERVER_GROUP_TYPE_FEATURE_SERVER_GROUP,
            options=options,
        )
        if isinstance(scaling_config, configs.AutoscalingConfig):
            args.autoscaling_config.CopyFrom(scaling_config._to_proto())
        elif isinstance(scaling_config, configs.ProvisionedScalingConfig):
            args.provisioned_scaling_config.CopyFrom(scaling_config._to_proto())
        else:
            msg = f"Invalid scaling config type: {type(scaling_config)}"
            raise TypeError(msg)
        info = base_tecton_object.TectonObjectInfo.from_args_proto(args.info, args.server_group_id)
        source_info = construct_fco_source_info(args.server_group_id)
        self.__attrs_init__(
            info=info,
            args=args,
            spec=None,
            source_info=source_info,
            scaling_config=scaling_config,
        )
        self._spec = specs.FeatureServerGroupSpec.from_args_proto(self._args)
        if not conf.get_bool("TECTON_SKIP_OBJECT_VALIDATION"):
            self._validate()
        base_tecton_object._register_local_object(self)


@attrs.define(eq=False)
class TransformServerGroup(BaseServerGroup):
    """
    Configuration used to specify the options for provisioning a server group for the transform server.
    """

    environment: str = attrs.field(repr=False)

    def __init__(
        self,
        *,
        name: str,
        scaling_config: Optional[Union[configs.AutoscalingConfig, configs.ProvisionedScalingConfig]],
        description: Optional[str] = None,
        tags: Optional[Dict[str, str]] = None,
        owner: Optional[str] = None,
        prevent_destroy: bool = False,
        environment: Optional[str] = None,
        options: Optional[Dict[str, str]] = None,
    ):
        """
        Instantiates a new TransformServerGroup.

        :param name: A unique name for the Transform Server Group.
        :param description: A human-readable description.
        :param tags: Tags associated with this Tecton Object (key-value pairs of arbitrary metadata).
        :param owner: Owner name (typically the email of the primary maintainer).
        :param prevent_destroy: If True, this Tecton object will be blocked from being deleted or re-created (i.e. a
            destructive update) during tecton plan/apply. To remove or update this object, `prevent_destroy` must be set to False
            via the same tecton apply or a separate tecton apply. `prevent_destroy` can be used to prevent accidental changes
            such as inadvertantly deleting a Feature Service used in production or recreating a Feature View that
            triggers expensive rematerialization jobs. `prevent_destroy` also blocks changes to dependent Tecton objects
            that would trigger a recreate of the tagged object, e.g. if `prevent_destroy` is set on a Feature Service,
            that will also prevent deletions or re-creates of Feature Views used in that service. `prevent_destroy` is
            only enforced in live (i.e. non-dev) workspaces.
        :param scaling_config: A configuration for creating an autoscaling or provisioned server group.
        :param environment: The name of the Python environment.
        :param options: Additional options to configure the Transform Server Group. Used for advanced use cases and beta features.
        """
        args = server_group_pb2.ServerGroupArgs(
            server_group_id=id_helper.IdHelper.generate_id(),
            info=basic_info_pb2.BasicInfo(name=name, description=description, tags=tags, owner=owner),
            prevent_destroy=prevent_destroy,
            transform_server_group_args=server_group_pb2.TransformServerGroupArgs(environment=environment),
            server_group_type=server_group_type_pb2.SERVER_GROUP_TYPE_TRANSFORM_SERVER_GROUP,
            options=options,
        )

        if isinstance(scaling_config, configs.AutoscalingConfig):
            args.autoscaling_config.CopyFrom(scaling_config._to_proto())
        elif isinstance(scaling_config, configs.ProvisionedScalingConfig):
            args.provisioned_scaling_config.CopyFrom(scaling_config._to_proto())
        else:
            msg = f"Invalid scaling config type: {type(scaling_config)}"
            raise TypeError(msg)
        info = base_tecton_object.TectonObjectInfo.from_args_proto(args.info, args.server_group_id)
        source_info = construct_fco_source_info(args.server_group_id)
        self.__attrs_init__(
            info=info,
            args=args,
            spec=None,
            source_info=source_info,
            scaling_config=scaling_config,
            environment=environment,
        )
        self._spec = specs.TransformServerGroupSpec.from_args_proto(self._args)
        if not conf.get_bool("TECTON_SKIP_OBJECT_VALIDATION"):
            self._validate()
        base_tecton_object._register_local_object(self)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/resource_provider__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.args import user_defined_function__client_pb2 as tecton__proto_dot_args_dot_user__defined__function____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.common import secret__client_pb2 as tecton__proto_dot_common_dot_secret____client__pb2
from tecton_proto.data import fco_metadata__client_pb2 as tecton__proto_dot_data_dot_fco__metadata____client__pb2
from tecton_proto.validation import validator__client_pb2 as tecton__proto_dot_validation_dot_validator____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n1tecton_proto/data/resource_provider__client.proto\x12\x11tecton_proto.data\x1a\x35tecton_proto/args/user_defined_function__client.proto\x1a$tecton_proto/common/id__client.proto\x1a(tecton_proto/common/secret__client.proto\x1a,tecton_proto/data/fco_metadata__client.proto\x1a/tecton_proto/validation/validator__client.proto\"\xea\x03\n\x10ResourceProvider\x12I\n\x14resource_provider_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x12resourceProviderId\x12\x41\n\x0c\x66\x63o_metadata\x18\x07 \x01(\x0b\x32\x1e.tecton_proto.data.FcoMetadataR\x0b\x66\x63oMetadata\x12J\n\x07secrets\x18\x03 \x03(\x0b\x32\x30.tecton_proto.data.ResourceProvider.SecretsEntryR\x07secrets\x12\x42\n\x08\x66unction\x18\x04 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\x08\x66unction\x12V\n\x0fvalidation_args\x18\t \x01(\x0b\x32-.tecton_proto.validation.ResourceProviderArgsR\x0evalidationArgs\x1a`\n\x0cSecretsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12:\n\x05value\x18\x02 \x01(\x0b\x32$.tecton_proto.common.SecretReferenceR\x05value:\x02\x38\x01\x42;\n\x0f\x63om.tecton.dataP\x01Z&github.com/tecton-ai/tecton_proto/data')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.resource_provider__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001Z&github.com/tecton-ai/tecton_proto/data'
  _RESOURCEPROVIDER_SECRETSENTRY._options = None
  _RESOURCEPROVIDER_SECRETSENTRY._serialized_options = b'8\001'
  _RESOURCEPROVIDER._serialized_start=303
  _RESOURCEPROVIDER._serialized_end=793
  _RESOURCEPROVIDER_SECRETSENTRY._serialized_start=697
  _RESOURCEPROVIDER_SECRETSENTRY._serialized_end=793
# @@protoc_insertion_point(module_scope)



from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from tecton_proto.common import id__client_pb2 as _id__client_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class FreshnessStatus(_message.Message):
    __slots__ = ["created_at", "expected_freshness", "feature_view_id", "feature_view_name", "freshness", "is_stale", "is_stream", "materialization_enabled"]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    EXPECTED_FRESHNESS_FIELD_NUMBER: _ClassVar[int]
    FEATURE_VIEW_ID_FIELD_NUMBER: _ClassVar[int]
    FEATURE_VIEW_NAME_FIELD_NUMBER: _ClassVar[int]
    FRESHNESS_FIELD_NUMBER: _ClassVar[int]
    IS_STALE_FIELD_NUMBER: _ClassVar[int]
    IS_STREAM_FIELD_NUMBER: _ClassVar[int]
    MATERIALIZATION_ENABLED_FIELD_NUMBER: _ClassVar[int]
    created_at: _timestamp_pb2.Timestamp
    expected_freshness: _duration_pb2.Duration
    feature_view_id: _id__client_pb2.Id
    feature_view_name: str
    freshness: _duration_pb2.Duration
    is_stale: bool
    is_stream: bool
    materialization_enabled: bool
    def __init__(self, feature_view_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., feature_view_name: _Optional[str] = ..., created_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., expected_freshness: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., is_stream: bool = ..., materialization_enabled: bool = ..., freshness: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., is_stale: bool = ...) -> None: ...



from tecton_proto.args import feature_service__client_pb2 as _feature_service__client_pb2
from tecton_proto.common import data_type__client_pb2 as _data_type__client_pb2
from tecton_proto.common import id__client_pb2 as _id__client_pb2
from tecton_proto.data import fco_metadata__client_pb2 as _fco_metadata__client_pb2
from tecton_proto.data import realtime_compute__client_pb2 as _realtime_compute__client_pb2
from tecton_proto.data import server_group__client_pb2 as _server_group__client_pb2
from tecton_proto.validation import validator__client_pb2 as _validator__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor
JOIN_KEY_BINDING_TYPE_BOUND: JoinKeyBindingType
JOIN_KEY_BINDING_TYPE_UNSPECIFIED: JoinKeyBindingType
JOIN_KEY_BINDING_TYPE_WILDCARD: JoinKeyBindingType

class FeatureService(_message.Message):
    __slots__ = ["enable_online_caching", "fco_metadata", "feature_server_group", "feature_service_id", "feature_set_items", "logging", "online_serving_enabled", "options", "realtime_environment", "transform_server_group", "validation_args"]
    class OptionsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    ENABLE_ONLINE_CACHING_FIELD_NUMBER: _ClassVar[int]
    FCO_METADATA_FIELD_NUMBER: _ClassVar[int]
    FEATURE_SERVER_GROUP_FIELD_NUMBER: _ClassVar[int]
    FEATURE_SERVICE_ID_FIELD_NUMBER: _ClassVar[int]
    FEATURE_SET_ITEMS_FIELD_NUMBER: _ClassVar[int]
    LOGGING_FIELD_NUMBER: _ClassVar[int]
    ONLINE_SERVING_ENABLED_FIELD_NUMBER: _ClassVar[int]
    OPTIONS_FIELD_NUMBER: _ClassVar[int]
    REALTIME_ENVIRONMENT_FIELD_NUMBER: _ClassVar[int]
    TRANSFORM_SERVER_GROUP_FIELD_NUMBER: _ClassVar[int]
    VALIDATION_ARGS_FIELD_NUMBER: _ClassVar[int]
    enable_online_caching: bool
    fco_metadata: _fco_metadata__client_pb2.FcoMetadata
    feature_server_group: _server_group__client_pb2.ServerGroup
    feature_service_id: _id__client_pb2.Id
    feature_set_items: _containers.RepeatedCompositeFieldContainer[FeatureSetItem]
    logging: _feature_service__client_pb2.LoggingConfigArgs
    online_serving_enabled: bool
    options: _containers.ScalarMap[str, str]
    realtime_environment: _realtime_compute__client_pb2.OnlineComputeConfig
    transform_server_group: _server_group__client_pb2.ServerGroup
    validation_args: _validator__client_pb2.FeatureServiceValidationArgs
    def __init__(self, feature_service_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., feature_set_items: _Optional[_Iterable[_Union[FeatureSetItem, _Mapping]]] = ..., fco_metadata: _Optional[_Union[_fco_metadata__client_pb2.FcoMetadata, _Mapping]] = ..., online_serving_enabled: bool = ..., logging: _Optional[_Union[_feature_service__client_pb2.LoggingConfigArgs, _Mapping]] = ..., validation_args: _Optional[_Union[_validator__client_pb2.FeatureServiceValidationArgs, _Mapping]] = ..., realtime_environment: _Optional[_Union[_realtime_compute__client_pb2.OnlineComputeConfig, _Mapping]] = ..., enable_online_caching: bool = ..., transform_server_group: _Optional[_Union[_server_group__client_pb2.ServerGroup, _Mapping]] = ..., feature_server_group: _Optional[_Union[_server_group__client_pb2.ServerGroup, _Mapping]] = ..., options: _Optional[_Mapping[str, str]] = ...) -> None: ...

class FeatureSetItem(_message.Message):
    __slots__ = ["feature_columns", "feature_view_id", "join_configuration_items", "namespace"]
    FEATURE_COLUMNS_FIELD_NUMBER: _ClassVar[int]
    FEATURE_VIEW_ID_FIELD_NUMBER: _ClassVar[int]
    JOIN_CONFIGURATION_ITEMS_FIELD_NUMBER: _ClassVar[int]
    NAMESPACE_FIELD_NUMBER: _ClassVar[int]
    feature_columns: _containers.RepeatedScalarFieldContainer[str]
    feature_view_id: _id__client_pb2.Id
    join_configuration_items: _containers.RepeatedCompositeFieldContainer[JoinConfigurationItem]
    namespace: str
    def __init__(self, feature_view_id: _Optional[_Union[_id__client_pb2.Id, _Mapping]] = ..., join_configuration_items: _Optional[_Iterable[_Union[JoinConfigurationItem, _Mapping]]] = ..., namespace: _Optional[str] = ..., feature_columns: _Optional[_Iterable[str]] = ...) -> None: ...

class JoinConfigurationItem(_message.Message):
    __slots__ = ["package_column_name", "spine_column_name"]
    PACKAGE_COLUMN_NAME_FIELD_NUMBER: _ClassVar[int]
    SPINE_COLUMN_NAME_FIELD_NUMBER: _ClassVar[int]
    package_column_name: str
    spine_column_name: str
    def __init__(self, spine_column_name: _Optional[str] = ..., package_column_name: _Optional[str] = ...) -> None: ...

class JoinKeyComponent(_message.Message):
    __slots__ = ["binding_type", "data_type", "spine_column_name"]
    BINDING_TYPE_FIELD_NUMBER: _ClassVar[int]
    DATA_TYPE_FIELD_NUMBER: _ClassVar[int]
    SPINE_COLUMN_NAME_FIELD_NUMBER: _ClassVar[int]
    binding_type: JoinKeyBindingType
    data_type: _data_type__client_pb2.DataType
    spine_column_name: str
    def __init__(self, spine_column_name: _Optional[str] = ..., binding_type: _Optional[_Union[JoinKeyBindingType, str]] = ..., data_type: _Optional[_Union[_data_type__client_pb2.DataType, _Mapping]] = ...) -> None: ...

class JoinKeyTemplate(_message.Message):
    __slots__ = ["components"]
    COMPONENTS_FIELD_NUMBER: _ClassVar[int]
    components: _containers.RepeatedCompositeFieldContainer[JoinKeyComponent]
    def __init__(self, components: _Optional[_Iterable[_Union[JoinKeyComponent, _Mapping]]] = ...) -> None: ...

class JoinKeyBindingType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
